#!/bin/bash
### BEGIN INIT INFO
# Provides:          start.sh
# Required-Start:    $remote_fs $syslog
# Required-Stop:     $remote_fs $syslog
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 
# Description:       /etc/init.d/sicunet.sh
### END INIT INFO

case "$1" in
start)

#---------------------------------------------------------------
# define
#---------------------------------------------------------------
SPIDER_BIN=/spider/bin
SPIDER_SBIN=/spider/sbin
SPIDER_SHELL=/spider/shell
SPIDER_DAEMON=/spider/daemon
SPIDER_DATABASE=/spider/database
SPIDER_WEB=/spider/web
SPIDER_CONF=/spider/conf
USR_LOCAL=/usr/local
TMP_DATABASE=/tmp/SpiderDB
VAR_LIGHTTPD=/var/run/lighttpd

#---------------------------------------------------------------
# setup.sh
#---------------------------------------------------------------
if [ ! -f "$SPIDER_SHELL/setup.done" ]; then
    $SPIDER_SHELL/setup.sh &
    exit 0
fi

#---------------------------------------------------------------
# stop.sh
#---------------------------------------------------------------
$SPIDER_SHELL/stop.sh

#---------------------------------------------------------------
# start.sh
#---------------------------------------------------------------
if [ ! -L "/etc/rc3.d/S99sicunet" ]; then
    ln -s $SPIDER_SHELL/start.sh /etc/init.d/sicunet.sh
    ln -s /etc/init.d/sicunet.sh /etc/rc3.d/S99sicunet
fi

#---------------------------------------------------------------
# NetworkManager
#---------------------------------------------------------------
systemctl stop NetworkManager.service
systemctl disable NetworkManager.service

#---------------------------------------------------------------
# ntp
#---------------------------------------------------------------
systemctl stop ntp
systemctl disable ntp

#---------------------------------------------------------------
# ntpdate
#---------------------------------------------------------------
if [ -f /usr/sbin/ntpdate ]; then
    mv /usr/sbin/ntpdate /usr/sbin/ntpdate_
fi

#---------------------------------------------------------------
# lighttpd account
#---------------------------------------------------------------
if [ ! -d "/home/lighttpd" ]; then
    adduser lighttpd
fi

#---------------------------------------------------------------
# vsftpd account
#---------------------------------------------------------------
if [ ! -d "/home/vsftpd" ]; then
    adduser vsftpd
fi

#---------------------------------------------------------------
# sudoers
#---------------------------------------------------------------
rm /etc/sudoers.d/*
usermod -L pi
usermod -L fa

#---------------------------------------------------------------
# daemon link
#---------------------------------------------------------------
rm -rf $USR_LOCAL/*
DIRECTORY_NAME=(curl
                libpcap
                libxml2
                lighttpd
                openssl
                pcre
                php
                sqlite
                vsftpd)
for dirName in "${DIRECTORY_NAME[@]}"; do
    if [ ! -L "$USR_LOCAL/$dirName" ]; then
        ln -s $SPIDER_DAEMON/$dirName $USR_LOCAL/$dirName
    fi
done

#---------------------------------------------------------------
# set uid
#---------------------------------------------------------------
chmod u+s $SPIDER_BIN/busybox
chmod u+s $SPIDER_SBIN/spider-cgi

#---------------------------------------------------------------
# binary copy
#---------------------------------------------------------------
$SPIDER_BIN/cp -fa $SPIDER_SBIN/spider-command /usr/bin

#---------------------------------------------------------------
# library copy
#---------------------------------------------------------------
$SPIDER_BIN/cp -fa $SPIDER_DAEMON/curl/lib/*.so* /usr/lib
$SPIDER_BIN/cp -fa $SPIDER_DAEMON/openssl/lib/*.so* /usr/lib

#---------------------------------------------------------------
# library path
#---------------------------------------------------------------
export LD_LIBRARY_PATH=$SPIDER_DAEMON/curl/lib:$SPIDER_DAEMON/libpcap/lib:$SPIDER_DAEMON/libxml2/lib:$SPIDER_DAEMON/openssl/lib:$SPIDER_DAEMON/pcre/lib:$SPIDER_DAEMON/sqlite/lib

#---------------------------------------------------------------
# passwd init
#---------------------------------------------------------------
$SPIDER_SBIN/spider-cgi passwd init

#---------------------------------------------------------------
# web check
#---------------------------------------------------------------
if [ ! -L "$SPIDER_WEB/webroot/help" ]; then
    ln -s $SPIDER_WEB/app/help  $SPIDER_WEB/webroot/help
fi
if [ ! -d "$VAR_LIGHTTPD" ]; then
    mkdir $VAR_LIGHTTPD
fi
chown -R lighttpd $SPIDER_WEB
chown -R lighttpd $SPIDER_CONF
chown -R lighttpd $USR_LOCAL/lighttpd/cert
chown -R lighttpd $VAR_LIGHTTPD

#---------------------------------------------------------------
# tmp check
#---------------------------------------------------------------
DIRECTORY_NAME=(backup-tmp
                restore-tmp
                updater-tmp
                install-tmp
                upload
                log)
for dirName in "${DIRECTORY_NAME[@]}"; do
    if [ -d "/tmp/$dirName" ]; then
        rm -rf /tmp/$dirName
    fi
    mkdir /tmp/$dirName
    chown -R lighttpd /tmp/$dirName
done

#---------------------------------------------------------------
# database copy
#---------------------------------------------------------------
if [ ! -d "$TMP_DATABASE" ]; then
    mkdir $TMP_DATABASE
fi
$SPIDER_SBIN/spider-dbback copy

#---------------------------------------------------------------
# database check
#---------------------------------------------------------------
DATABASE_FILE=(Spider.db
               Network.db
               SpiderLog.db)
for fileName in "${DATABASE_FILE[@]}"; do
    if [ ! -f "$TMP_DATABASE/$fileName" ]; then
        $SPIDER_BIN/cp -fa $SPIDER_DATABASE/$fileName $TMP_DATABASE
    fi
done
chown -R lighttpd $TMP_DATABASE
chown -R lighttpd $SPIDER_DATABASE

#---------------------------------------------------------------
# database migration
#---------------------------------------------------------------
$SPIDER_SBIN/spider-migration

#---------------------------------------------------------------
# database optimization
#---------------------------------------------------------------
$SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/Spider.db "VACUUM"
$SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/Network.db "VACUUM"
$SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/SpiderLog.db "VACUUM"

#---------------------------------------------------------------
# nameserver
#---------------------------------------------------------------
echo "nameserver 8.8.8.8" > /etc/resolv.conf

#---------------------------------------------------------------
# network init
#---------------------------------------------------------------
$SPIDER_SBIN/spider-cgi nconf init

#---------------------------------------------------------------
# vsftpd start
#---------------------------------------------------------------
killall vsftpd
$USR_LOCAL/vsftpd/vsftpd $USR_LOCAL/vsftpd/vsftpd.conf

#---------------------------------------------------------------
# HTTP/HTTPS
#---------------------------------------------------------------
IFS=','
row=( $($SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/Network.db -separator "," "SELECT IFNULL(Https,0), IFNULL(HttpPort,80), IFNULL(HttpsPort,443) FROM Network") )
if [ ${row[0]} -eq 1 ]; then
    sed 's/${port}/'"${row[2]}"'/' $USR_LOCAL/lighttpd/lighttpsd.conf > $USR_LOCAL/lighttpd/current.conf
else
    sed 's/${port}/'"${row[1]}"'/' $USR_LOCAL/lighttpd/lighttpd.conf > $USR_LOCAL/lighttpd/current.conf
fi

#---------------------------------------------------------------
# OPENAPI
#---------------------------------------------------------------
IFS=','
row=( $($SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/Spider.db -separator "," "SELECT IFNULL(Enabled,0), IFNULL(Port,0), IFNULL(ClientIP,''), IFNULL(AuthKey,''), IFNULL(HashKey,''), IFNULL(AuthKeyUse,'') FROM OPENAPI") )
if [ ${row[0]} -eq 1 ]; then
    spider_model=( $($SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/Network.db -separator "," "SELECT IFNULL(Model,0), IFNULL(Type,0) FROM Network") )
    export OPENAPI_DATA=${row[2]}:${row[3]}:${row[4]}:${row[5]}:${spider_model[0]}:${spider_model[1]}
    sed -i 's/${apiport}/'"${row[1]}"'/' $USR_LOCAL/lighttpd/current.conf
    sed -i 's/#{ONAPI}/''/' $USR_LOCAL/lighttpd/current.conf
fi

#---------------------------------------------------------------
# sicunet.crt
#---------------------------------------------------------------
if [ ! -f $USR_LOCAL/lighttpd/cert/sicunet.crt ]; then
    $SPIDER_BIN/cp -fa $USR_LOCAL/lighttpd/cert/sicunet.crt.bak $USR_LOCAL/lighttpd/cert/sicunet.crt
fi

#---------------------------------------------------------------
# lighttpd start
#---------------------------------------------------------------
$USR_LOCAL/lighttpd/sbin/lighttpd -f $USR_LOCAL/lighttpd/current.conf

#---------------------------------------------------------------
# update server
#---------------------------------------------------------------
$SPIDER_DAEMON/sqlite/bin/sqlite3 $TMP_DATABASE/Spider.db "UPDATE NetworkInfo SET UpdateAddress='update.remote-manager.net', UpdatePort='2021', UpdateID='neptune', UpdatePassword='neptune'"

#---------------------------------------------------------------
# timezone
#---------------------------------------------------------------
$SPIDER_BIN/cp -fa /usr/share/zoneinfo/UTC /etc/localtime

#---------------------------------------------------------------
# time set
#---------------------------------------------------------------
chmod 777 /opt
if [ -f /opt/savedate ]; then
    date -s "$(cat /opt/savedate)"
#    rm /opt/savedate
fi

$SPIDER_SBIN/spider-cgi time set

#---------------------------------------------------------------
# bak program
#---------------------------------------------------------------
$SPIDER_BIN/cp -fa $SPIDER_SBIN/spider-updater.bak $SPIDER_SBIN/spider-updater
$SPIDER_BIN/cp -fa $SPIDER_SBIN/spider-monitor.bak $SPIDER_SBIN/spider-monitor
$SPIDER_BIN/cp -fa $SPIDER_SBIN/spider-watchdog.bak $SPIDER_SBIN/spider-watchdog

#---------------------------------------------------------------
# spider start... 
#---------------------------------------------------------------
$SPIDER_SBIN/spider-e &
$SPIDER_SBIN/spider-fifo &
$SPIDER_SBIN/spider-install &
$SPIDER_SBIN/spider-dbback &
$SPIDER_SBIN/spider-updater &
$SPIDER_SBIN/spider-node &
$SPIDER_SBIN/spider-cloud &
$SPIDER_SBIN/spider-credential &
$SPIDER_SBIN/spider-logapi &
$SPIDER_SBIN/spider-nvr &
$SPIDER_SBIN/spider-monitor &

exit 0
;;

esac
exit 0
