<?php
/***********************************************************************************
 * FILE NAME : ConstOpenapiErrorCode.php                         DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2018.12.26 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 *        2    0.0.1       2019.05.15  CJMOON     ADD Code 0011, 0012, 0050 
 *        3    0.0.2       2019.12.04  CJMOON     ADD Code 0004
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : OpenAPI Error Code List
 *  
 ***********************************************************************************/
class ConstOpenapiErrorCode
{
    const ERR_SUCCESS = '0000';              /* 정상 */

    const ERR_AUTHKEY_VALUE = '0001';        /* AuthKey Value가 NULL */
    const ERR_AUTHKEY_NOT_MATCH = '0002';    /* AuthKey Value가 불일치 */
    const ERR_AUTH_FAIL = '0003';            /* 인증실패 */
    const ERR_LOGIN_FAIL = '0004';           /* Login Failed */
    
    const ERR_INPUT_VALUE    = '0010';            /* Input Value가 잘못됨 */
    const ERR_INPUT_REQUIRED_0011 = '0011';       /* User 에서 이름 항목 중 필수 누락 */
    const ERR_INPUT_REQUIRED_0012 = '0012';       /* Image File Upload Failed */
    const ERR_INPUT_REQUIRED_0013 = '0013';       /* 필수 항목 Key(No) 누락 */
    const ERR_INPUT_REQUIRED_0014 = '0014';       /* 필수 항목 Card Format 누락 */
    const ERR_INPUT_REQUIRED_0015 = '0015';       /* 필수 항목 CardNo(카드번호) 누락 */
    const ERR_INPUT_REQUIRED_0016 = '0016';       /* 필수 항목 Access Level List 누락 */
    const ERR_INPUT_REQUIRED_0017 = '0017';       /* 필수 항목 ActDate, ExpDate Value 문제 */
    const ERR_INPUT_REQUIRED_0018 = '0018';       /* Access Level에서 사용중인 스케쥴 */

    const ERR_CLIENT_IS_ALIVE = '0020';           /* Client의 상태가 ALive 상태가 아님 */
    const ERR_CLIENT_NOT_ALIVE = '0021';          /* Client의 상태가 Not ALive 상태가 아님 */

    const ERR_DOOR_CONTROL_NOT_NORMAL = "0030";   /* Door의 상태가 Normal Mode 상태가 아님 */
    const ERR_DOOR_CONTROL_MANTRAP = "0031";      /* Door의 상태가 Mantrap Mode On 상태 임 */
    const ERR_DOOR_CONTROL_NOT_ACTION_MODE_MODE = "0032"; /* Door의 알수 없는 수동동작 명령 */

    const ERR_ACCOUNT_ID_DUPLICATION = "0040";    /* User ID 중복 */
    const ERR_LIMITED_CARD_HOLDER    = "0050";    /* Card Holder 최대치 초과 */
    const ERR_LIMITED_CARD           = "0051";    /* Card 최대치 초과 */
    const ERR_LIMITED_ACCESS_LEVEL   = "0052";    /* Access Level 최대치 초과 */
    const ERR_LIMITED_USERCARD_ALLOCATION= "0053";/* User당 최대 할당 가능 최대치 초과 */ 
    const ERR_LIMITED_THREAT_LEVEL = "0054";      /* Threat Level 최대치 초과 */
    const ERR_LIMITED_SCHEDULE = "0056";          /* Schedule 최대치 초과 */
    const ERR_DUPLECATION_CARD = "0070";          /* 중복 카드 여부 확인 */
    
    const ERR_INPUT_VALUE_MISMATCH = "9996";      /* UPDATE시 입력된 데이타와 기존 데이타 불일치 여부 */
    const ERR_MAXSEARCH_DAYS = '9997';            /* 최대 검색 가능 기간 초과 */    
    const ERR_SQLINJECTION = '9998';              /* Input Value에 SQL Injection 의심코드 존재 */
    const ERR_ETC= '9999';   /* 기타 오류 */

    /* ================================================================== */
    static function getErrorMessage($errCode) 
    {
        switch($errCode) {
            case self::ERR_SUCCESS : return ("Success"); 
            case self::ERR_AUTHKEY_VALUE : return ("AuthKey Value Error"); 
            case self::ERR_AUTHKEY_NOT_MATCH : return ("AuthKey Not Match"); 
            case self::ERR_AUTH_FAIL : return ("Authentication failed");
            case self::ERR_LOGIN_FAIL : return ("Login failed");
        
            case self::ERR_INPUT_VALUE  : return ("Input Value Validation Error"); 
            case self::ERR_INPUT_REQUIRED_0011 : return ("Input Value Required Error (FirstName, lastName)");
            case self::ERR_INPUT_REQUIRED_0012 : return ("Image File Upload Failed");
            case self::ERR_INPUT_REQUIRED_0013 : return ("Input Value Required Error (No or UserNo Field)");
            case self::ERR_INPUT_REQUIRED_0014 : return ("Input Value Required Error (CardFormat)");
            case self::ERR_INPUT_REQUIRED_0015 : return ("Input Value Required Error (CardNo)");
            case self::ERR_INPUT_REQUIRED_0016 : return ("Input Value Required Error (AccessLevelList)");
            case self::ERR_INPUT_REQUIRED_0017 : return ("Input Value Required Error (ActDate or ExpDate)");
            case self::ERR_INPUT_REQUIRED_0018 : return ("Input Value Required Error (Schedule used at access level)");

            case self::ERR_CLIENT_NOT_ALIVE : return ("Client Not ALive Status"); 
            case self::ERR_CLIENT_IS_ALIVE  : return ("Client Is ALive Status"); 
            
            case self::ERR_DOOR_CONTROL_NOT_NORMAL : return ("Error Door Control - Door is not Normal Mode"); 
            case self::ERR_DOOR_CONTROL_MANTRAP : return ("Error Door Control - Door is on Mantrap Mode"); 
            case self::ERR_DOOR_CONTROL_NOT_ACTION_MODE_MODE : return ("Error Door Control - Unknown Action Mode"); 
            case self::ERR_ACCOUNT_ID_DUPLICATION : return ("Error Account ID Duplication");
            case self::ERR_LIMITED_CARD_HOLDER : return ("The number of entries for Card Holder has been exceeded.");
            case self::ERR_LIMITED_CARD : return ("The number of entries for Card has been exceeded.");
            case self::ERR_LIMITED_ACCESS_LEVEL : return ("The number of entries for Access Level has been exceeded.");
            case self::ERR_LIMITED_USERCARD_ALLOCATION : return ("The number of entries for User Card has been exceeded.");            
            case self::ERR_LIMITED_THREAT_LEVEL : return ("The number of entries for Threat Level has been exceeded");
            case self::ERR_LIMITED_SCHEDULE : return ("The number of entries for 'Schedule' has been exceeded");
            case self::ERR_DUPLECATION_CARD : return("This card number already exist. Card numbers cannot be duplicated");
            
            case self::ERR_INPUT_VALUE_MISMATCH : return ("Input Value Required Mismatch Error");
            case self::ERR_MAXSEARCH_DAYS : return ("Error Maximum search period exceeded");
            case self::ERR_SQLINJECTION : return ("Input Value is SQL Injection Code");             

            case self::ERR_ETC : return ("Etc Error Code");        
            default : return ("Etc Error Code1");
        }
    }
    
}
?>