<?php
/***********************************************************************************
 * FILE NAME : OpenApiBaseImpl.php                               DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2018.12.26 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 *        2    0.0.2       2019.05.15  CJMOON     ADD SQL Injection & Paging function
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : OpenAPI abstract
 *  
 ***********************************************************************************/
require_once APP_DIR.'/libraries/ConstTable.php';
require_once APP_DIR.'/libraries/Input.php';
require_once APP_DIR.'/libraries/Util.php';
require_once APP_DIR.'/libraries/Log.php';

abstract class OpenApiBaseImpl
{
    public $conn;
    public $conn_log;
    public $net_work;

    public $neptune_model;
    public $neptune_type;
    
    public $request_header;
    
    public $input;
    public $log;
    public $util;

    const STR_ERRORCODE = 'errCode';
    const STR_ERRORMSG = 'errMsg';
    const STR_DATA = 'data';

    /* ================================================================== */    
    /* CLASS 생성자 */    
    public function __construct() {
        $this->conn         = $GLOBALS['conn'];
        $this->conn_log     = $GLOBALS['conn_log'];
        $this->net_work     = $GLOBALS['net_work'];
        
        $this->request_header = getallheaders();
        
        $this->input = new Input();
        $this->log = new Log();
		$this->util = new Util();
		
		if(empty($this->request_header['Site'])){
	        $_SESSION['spider_site'] = OPENAPI_SITE;
        }else{
	        $_SESSION['spider_site'] = $this->request_header['Site'];
	    }
	    
	    $_SESSION['spider_userno'] = '';
	    $_SESSION['spider_id'] = '';

        self::init();
        if(self::check_parameter()) {
            self::toError(ConstOpenapiErrorCode::ERR_SQLINJECTION);
            exit;
        }
    }
    
    /* ================================================================== */    
    /* 소멸자 */    
    public function __destruct() {         
    }
    
    /* ================================================================== */
    /* 내부 초기화 */
    public function init() {
        $openapi_data = $_ENV["OPENAPI_DATA"];
        $openapi_data = explode(":", $openapi_data);
        $clientIP = $openapi_data[0];
        $AuthKey = $openapi_data[1];
        $HashKey = $openapi_data[2];
        $AuthKeyUse = $openapi_data[3];
        $this->neptune_model = $openapi_data[4];
        $this->neptune_type = $openapi_data[5];
        
        if($AuthKeyUse === "1") self::check_auth($clientIP, $AuthKey, $HashKey);
        else if($AuthKeyUse === "2") self::check_session($AuthKey);
    }

    /* ================================================================== */
    /* Set Auth Key */    
    public function setAuthKey($setAuthkey)
    {
        $openapi_data = $_ENV["OPENAPI_DATA"];
        $openapi_data = explode(":", $openapi_data);
        $clientIP = $openapi_data[0];
        $AuthKey = $openapi_data[1];
        $HashKey = $openapi_data[2];
        $AuthKeyUse = $openapi_data[3];
        
        putenv("OPENAPI_DATA=$clientIP:$setAuthkey:$HashKey:$AuthKeyUse:$this->neptune_model:$this->neptune_type");        
    }

    /* ================================================================== */
    /* Hash Key Return */
    public function getHashKey()
    {
        $openapi_data = $_ENV["OPENAPI_DATA"];
        $openapi_data = explode(":", $openapi_data);
        //$clientIP = $openapi_data[0];
        //$AuthKey = $openapi_data[1];
        $HashKey = $openapi_data[2];
        //$AuthKeyUse = $openapi_data[3];

        return ($HashKey);
    }
    /* ================================================================== */
    /* JSON Format 출력 */
    public function toJson($errCode, $output=NULL) {                
        if(empty($output)) {
            echo json_encode(array(self::STR_ERRORCODE => $errCode,
                                self::STR_ERRORMSG  => ConstOpenapiErrorCode::getErrorMessage($errCode)));            
        }
        else {        
            echo json_encode(array(self::STR_ERRORCODE => $errCode,
                                self::STR_ERRORMSG  => ConstOpenapiErrorCode::getErrorMessage($errCode),
                                self::STR_DATA      => $output));
        }
       
       /*
            echo json_encode(array(self::STR_ERRORCODE => $errCode,
                                self::STR_ERRORMSG  => ConstOpenapiErrorCode::getErrorMessage($errCode),
                                self::STR_DATA      => $output));       
        */                                
    }

    /* ================================================================== */
    /* JSON Format 출력 */
    public function toError($errCode, $errMsg=NULL) {
	    
        if(empty($errMsg)) {
	        $requestData = array(
	        	self::STR_ERRORCODE => $errCode,
                self::STR_ERRORMSG  => ConstOpenapiErrorCode::getErrorMessage($errCode)
            );
                       
        }else {
            $requestData = array(
            					self::STR_ERRORCODE => $errCode,
								self::STR_ERRORMSG  => $errMsg
							);
        }
        
        echo json_encode($requestData); 
    }

    /* ================================================================== */
    /** SQL Injection 검증을 위한 Web Parameter 확인 
     *  TRUE : SQL Injection
     * FALSE : SQL Injection Check Success
     */
    public function check_parameter() {
        
        if($_SERVER['REQUEST_METHOD'] === "PUT" || 
           $_SERVER['REQUEST_METHOD'] === "DELETE") 
        {
            parse_str(file_get_contents("php://input"), $PutParams);
            foreach( $PutParams as $key=>$val ) {
                if(self::IS_SQLInjection($val)) {
                    return TRUE;
                }
            }

            return;
        }
        
        if(count($_REQUEST) == 0) return FALSE;

        foreach( $_REQUEST as $key=>$val ) {
            if(self::IS_SQLInjection($val)) {
                return TRUE;
            }
        }
    }

    /* ================================================================== */
    /* SQL Injection Check */
    public function IS_SQLInjection($v)
    {
        $UserInput = preg_replace("/[\r\n\s\t\’\;\”\=\-\-\#\/*]+/","", $v);
        //if(preg_match('/(union|select|from|where|update|delete|or)/i', $UserInput)) {
        //if(preg_match('/(union|select|from|where|update|delete)/i', $UserInput)) {
		if(preg_match('/( union | select | from | where | update | delete )/i', $UserInput)) {
				return TRUE;
        }

        return FALSE;
    }

    /* ================================================================== */
    public function check_session($AuthKey) 
    {
        if(empty($this->request_header['Auth-Key']) || 
            strncmp($this->request_header['Auth-Key'], $AuthKey, strlen($AuthKey)) != 0) {
            self::toError(ConstOpenapiErrorCode::ERR_AUTHKEY_VALUE);
            exit;
        }        
    }

    /* ================================================================== */
    public function check_auth($ClientIP, $AuthKey, $HashKey) 
    {
        if(empty($this->request_header['Auth-Key']) || 
           strncmp($this->request_header['Auth-Key'], $AuthKey, strlen($AuthKey)) != 0) {
            self::toError(ConstOpenapiErrorCode::ERR_AUTHKEY_VALUE);
            exit;
        }
                
        $authKeyValue = hash_hmac("sha256", $_SERVER['REMOTE_ADDR'], $HashKey);
        if(strncmp($this->request_header['Auth-Key'], $authKeyValue, strlen($authKeyValue)) != 0) {
            self::toError(ConstOpenapiErrorCode::ERR_AUTH_FAIL);
            exit;            
        }

        if(strncmp($_SERVER['REMOTE_ADDR'],$ClientIP, strlen($ClientIP)) != 0) {
            self::toError(ConstOpenapiErrorCode::ERR_AUTH_FAIL);
            exit;                        
        }
    }

    /* ================================================================== */
    /* OpenAPI 요청한 SITE 정보 추출 */
    public function getAccountSite() {
        if(empty($this->request_header['Site'])){
	        return (OPENAPI_SITE);
        }else{
	        return $this->request_header['Site'];
	    }
    }

    /* ================================================================== */
    /* Array to String Debuger 
       Example : echo parent::getArrayToString($list);
    */
    public function getArrayToString($dumpArray) {
        return print_r($dumpArray, TRUE); 
    }

    /* ================================================================== */
    /** 현재 페이지(pages) 기준으로 record row offset 계산
     * 
     */
    public function getCurrentPaging($pages, $pageRow=MAX_LIST_ROW) {
        return (($pages-1) * $pageRow);
    }
    /* ================================================================== */
    /* Paging 계산 Method                  
       Example : parent::getPaging(255, 20) List당 20개의 Row로 구성
    */
    public function getPaging($totalRow, $pageRow=MAX_LIST_ROW) {
      if(($totalRow % $pageRow) == 0) return intval($totalRow/$pageRow);
      else return intval(($totalRow/$pageRow)+1);
    }

    /* ================================================================== */
    /* Input Value Check 및 Default Value로 변환
       Example : 
    */    
    public function InputDef($in, $def=NULL) {

      if(empty($in) || $in === ''|| strlen($in) == 0) {
          if($def === '') return '';
           else return $def;
      }
      return $in;
    }

    /* DEBUG LINE
    public function get_function_origin_path($include_filename = true) {
		$bt = debug_backtrace();
		array_shift($bt);
		if ( array_key_exists(0, $bt) && array_key_exists('file', $bt[0]) ) {
			$file_path = $bt[0]['file'];
			if ( $include_filename === false ) {
				$file_path = str_replace(basename($file_path), '', $file_path);
			}
		} else {
			$file_path = null;
		}
		return $file_path;
	}
    */
}
?>