<?php
/***********************************************************************************
 * FILE NAME : AccessLevel.php                                          DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.01.03 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 *        1    0.0.2       2018.06.10  CJMOON     Bugfix
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Client API
 *  
 ***********************************************************************************/
include_once "./common/OpenApiBaseImpl.php";
include_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class AccessLevel extends OpenApiBaseImpl
{       
    /* ================================================================== */    
    /* AccessLevel Process */
   public function clist() 
   {
        try {	        
            $arr_schedule       = $this->to_array_schedule();

            $dbContext = $this->conn->prepare("SELECT No, Name, ScheduleNo, SelectType FROM AccessLevel WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite()));
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

            if(count($dbContext) > 0) {
                foreach( $dbContext as $key=>$val ) {
//                    $dbContext[$key]['ScheduleName'] = $arr_schedule[$val['ScheduleNo']];
//                    $dbContext[$key]['SelectType'] = EnumTable::$attrGroup[$val['SelectType']];

                    $dbContext[$key]['DoorList'] = self::to_list_AccessLevelReader($val['No']);                          
                }

                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($dbContext));       
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }                     
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch 
    }

    /* ================================================================== */
    /* 신규 Access Level Input */
    public function insert()
    {
        $Name       = strip_tags(trim($this->input->post('Name')));
        $Mean       = strip_tags($this->input->post('Mean'));
        $ScheduleNo = $this->input->post('ScheduleNo');
        $SelectType = $this->input->post('SelectType');
        $DoorList   = array_filter(explode(',',$this->input->post('DoorList')));
        
    try {        
            if(empty($Name) || $Name === '' || 
            $ScheduleNo ==='' || !is_numeric($ScheduleNo) || empty($ScheduleNo) || 
            $SelectType ==='' || !is_numeric($SelectType) || empty($SelectType) || 
            count($DoorList) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;             
            }

            /* 라이센스에 따른 최대 사용 가능 갯수 확인 */
            if($this->checkMaxLimitedSchedule()) {            
                parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_ACCESS_LEVEL);
                exit;            
            }

            /* User Key Create */
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM AccessLevel WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                
            if(!$dbContext->execute(array(parent::getAccountSite()))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $No = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */
            $valueArr = array(parent::getAccountSite(), $No, $Name, $Mean, $ScheduleNo, $SelectType);
            $dbContext    = $this->conn->prepare("INSERT INTO AccessLevel (Site,No,Name,Mean,ScheduleNo,SelectType) VALUES (?,?,?,?,?,?)");
                                    
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                            
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            } 

            $dbContext = null; /* Database Lock Free */

            self::insertAccessLevelReader($No, $DoorList);

            $this->log->set_log_device('alevel', 'insert', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("No"=>$No));     
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch  
                
    }

    /** ==================================================================
     *  Access Level Update
     */    
    public function update() {
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No         = $PutParams['No'];
            $Name       = strip_tags(trim($PutParams['Name']));
            $Mean       = strip_tags($PutParams['Mean']);
            $ScheduleNo = $PutParams['ScheduleNo'];
            $SelectType = $PutParams['SelectType'];
            $DoorList   = array_filter(explode(',',$PutParams['DoorList']));
            
            if(empty($No) || !is_numeric($No) || $No === '') {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }

            if(empty($Name) || $Name === '' ||
               $ScheduleNo ==='' || !is_numeric($ScheduleNo) || empty($ScheduleNo) || 
               $SelectType ==='' || !is_numeric($SelectType) || empty($SelectType) || 
               count($DoorList) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;             
            }

            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM AccessLevel WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchAll();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            }

            $valueArr = array($Name, $Mean, $ScheduleNo, $SelectType, parent::getAccountSite(), $No);
            $dbContext = $this->conn->prepare("UPDATE AccessLevel SET Name=?,Mean=?,ScheduleNo=?,SelectType=? WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

            /* 기존 Access Level Reader 삭제 */
            self::deleteAccessLevelReader($No);
            /* 새로운 Access Level Reader로 재입력 */
            self::insertAccessLevelReader($No, $DoorList);  

            $this->log->set_log_device('alevel', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch       

    }

    /** ==================================================================
     *  Access Level Delete
     */        
    public function delete() {
        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        /* User No Check Valid */
        if(empty($No) || !is_numeric($No) || $No === '') {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM AccessLevel WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 

            /* 기존 Access Level 삭제 */
            self::deleteAccessLevelReader($No);

            $dbContext = $this->conn->prepare("DELETE FROM AccessLevel WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = null; /* Database Lock Free */

            $this->log->set_log_device('alevel', 'delete', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch

    }    
   /** ==================================================================
     * Access Level Reader Insert
     * INSERT, UPDATE 공용 으로 사용.
     */
    private function insertAccessLevelReader($AccessLevelNo, $DoorList) {
        try {
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM AccessLevelReader WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                            
            $dbContext->execute(array(parent::getAccountSite()));
            $No = $dbContext->fetchColumn(); 

            $dbContext = $this->conn->prepare("INSERT INTO AccessLevelReader (Site, No, AccessLevelNo, ReaderNo) VALUES (?,?,?,?)");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            foreach( $DoorList as $value ) {                
                $valueArr = array(parent::getAccountSite(), $No, $AccessLevelNo, $value);         
                if(!$dbContext->execute($valueArr)) {
                    throw new Exception($this->conn->errorInfo()[2]);
                }

                $No = $No+1;
            }
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
    }

    /** ==================================================================
     *  Access Level Reader를 삭제 시 사용 (Update 시에도 사용 - 기존 데이타 삭제)
     */
    private function deleteAccessLevelReader($AccessLevelNo) {
        try {
            $dbContext = $this->conn->prepare("DELETE FROM AccessLevelReader WHERE Site=? AND AccessLevelNo=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $AccessLevelNo));
            $dbContext = null; /* Database Lock Free */
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                
    }

    /* ================================================================== */   
    /* Unlock 대상 List 추출 */
    private function  to_list_AccessLevelReader($AccessLevelNo) {
        $list   = $this->conn->prepare("SELECT ReaderNo FROM AccessLevelReader WHERE Site=? AND AccessLevelNo=?");
        $list->execute(array(parent::getAccountSite(), $AccessLevelNo));
        $list = $list->fetchAll(PDO::FETCH_COLUMN);
        $UnlockList = implode(", ", $list);
        
        return $UnlockList;
    }

    /* ================================================================== */    
    private function to_array_schedule()
    {        
        $list   = $this->conn->prepare("SELECT * FROM Schedule WHERE Site=?");
        $list->execute(array(parent::getAccountSite()));
        $list = $list->fetchAll(PDO::FETCH_ASSOC);

        $arr    = array();
        foreach ($list as $key=>$val)
        {
            $arr[$val['No']] = $val['Name'];
        }

        return $arr;
    }

    /** ==================================================================
     * License에 따른 최대 Access Level 확인
     * TRUE : 최대 Access Level초과
     * FALSE : 최대 Access Level 이하
     */
    private function checkMaxLimitedSchedule() {
        $maxAccessLevel = EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][11];
        if($this->util->GetRecordCount('AccessLevel') >= $maxAccessLevel) return TRUE;
        else FALSE;   
     }    

}
?>