<?php
/***********************************************************************************
 * FILE NAME : Card.php                                          DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.05.17 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.05.17  CJMOON     Init 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Card API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class Card extends OpenApiBaseImpl
{
    /* ================================================================== */    
    /* UserNo에 따른 Card List 정보 */
    public function clist()
    {
                

        try {
            $No = strip_tags(trim($this->input->get('No')));

            /* User No Check Valid */
            if(empty($No) || !is_numeric($No)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }

            /* ReaderNo : Card Scan 했을 당시의 Reader 번호 임 */
            $dbContext = $this->conn->prepare("SELECT No, CardNo, CardFormatNo, CardStatus, KeyNumber, SelectType, NeverExpire, Reason, ".
                                              "IFNULL(strftime('%Y%m%d',date(ActDate,'unixepoch')),'') ActDate , ".
                                              "IFNULL(strftime('%Y%m%d',date(ExpDate,'unixepoch')),'') ExpDate , ReaderNo, CardType  FROM Card WHERE Site=? AND UserNo=?");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                
            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);


            //SELECT AccessLevelNo FROM CardAccessLevel WHERE Site=? AND No=? AND CardNo=?

            foreach( $dbContext as $key=>$val )
            {
                $name = $this->conn->prepare("SELECT AccessLevelNo FROM CardAccessLevel WHERE Site=? AND CardNo=?");
                $name->execute(array(parent::getAccountSite(),$val['No']));
                $name = $name->fetchAll(PDO::FETCH_ASSOC);
                $AlList = implode(", ", $name);
                
                $val['AccessLevel'] = self::to_list_AccessLevel($val['No']);
                $dbContext[$key] = $val;

                //$val['AccessLevelNo']    = $AlList;
                //$dbContext[$key] = $val;
                //$dbContext[$key]['AccessLevelNo'] = self::to_list_AccessLevelReader($val['No']);
            }

            if(count($dbContext) > 0) {
                $vars = array( "total"=>count($dbContext), 'rows' => $dbContext);
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($vars));
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /* ================================================================== */    
    /* UserNo에 따른 Card 추가 ( ReaderNo 필드는 사용하지 않음. 
     *                          Card Scan시 ReaderNo용 이며, API는 미지원)
     */
    public function insert() { 

        $UserNo         = $this->input->post('UserNo', 0);        
        $CardNo         = $this->input->post('CardNo');
        $CardFormatNo   = $this->input->post('CardFormatNo', 0);
        $CardStatus     = $this->input->post('CardStatus', 0); /* Default */
        $CardType       = $this->input->post('CardType', 0);   /* Default */
        $KeyNumber      = $this->input->post('KeyNumber', 0);
        $NeverExpire    = $this->input->post('NeverExpire', 0); 
        $Reason         = strip_tags($this->input->post('Reason'));
        $ActDate        = self::to_timestamp($this->input->post('ActDate'));
        $ExpDate        = self::to_timestamp($this->input->post('ExpDate'), 23, 59, 59);
		$SelectType     = $this->input->post('SelectType', 1);
        $AccessLevel	= array_filter(explode(',',$this->input->post('AccessLevel')));
        
        /* User No Check Valid */
        if(empty($UserNo) || !is_numeric($UserNo)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        /* Card Format Check Valid */
        $cardFormat = $this->checkCardFormat($CardFormatNo);

        if($CardFormatNo == 0 || is_bool($cardFormat)) {            
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0014);
            exit;
        }

        $maxCardNumberLength = pow(2, $cardFormat) - 1;

         /* Card number Check Valid */
        if(empty($CardNo) || ((int)$CardNo > (int)$maxCardNumberLength) || !is_numeric($CardNo)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0015);
            exit;            
        }
    
        /* Access Level List Check Valid */
        if(count($AccessLevel) == 0) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0016);
            exit;
        }

        if(count($AccessLevel) > EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][3]) {
            parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_ACCESS_LEVEL);
            exit;
        }

        /* Limited Card Counter */
        if(self::checkMaxLimitedCard()) {            
            parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_CARD);
            exit;
        }

        /* Max User Card Allocation Counter */
        if(self::checkMaxUserCardAllocation($UserNo)) {
            parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_USERCARD_ALLOCATION);
            exit;
        }

        if(self::checkDuplicationCard($CardNo, $CardFormatNo )) {
            parent::toError(ConstOpenapiErrorCode::ERR_DUPLECATION_CARD);
            exit;
        }

        /* NeverExpire가 0 이면 ActDate와 ExpDate 기준으로 처리 (Card.php Logic 기준)
        *  CardType:5(One time) 이면 
        */
        if($CardStatus === '0' &&  $CardType === '5') {
            $ActDate = ''; $ExpDate=''; $NeverExpire='';
        } 
        else {
            if( empty($ActDate) && $NeverExpire == 0 ) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0017);
                exit;
            }

            if( $NeverExpire == 0 ) {
                if( !empty($ExpDate) && $ActDate > $ExpDate ) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0017);
                    exit;
                }
            } else {
                $ActDate	= '';
                $ExpDate	= '';
            }
        }

         /* User Key Create */
         $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM Card WHERE Site=?");
         if(is_bool($dbContext)) {
             throw new Exception($this->conn->errorInfo()[2]);
         }
                         
         $dbContext->execute(array(parent::getAccountSite()));
         $No = $dbContext->fetchColumn();  
         $dbContext = null; /* Database Lock Free */

         $valueArr = array(parent::getAccountSite(), $No, $CardNo, $CardFormatNo, $CardStatus,
                           $KeyNumber, $SelectType, $NeverExpire, $Reason, $ActDate, $ExpDate,
                           $UserNo, $CardType);

         $dbContext = $this->conn->prepare("INSERT INTO Card (Site, No, CardNo, CardFormatNo, ".
                                   " CardStatus, KeyNumber, SelectType, NeverExpire, Reason, ".
                                   " ActDate, ExpDate, UserNo, ReaderNo, CardType) ".
                                   " VALUES (?,?,?,?,?,?,?,?,?,?,?,?,0,?)");

        if(is_bool($dbContext)) {
            throw new Exception($this->conn->errorInfo()[2]);
        }
            
        if(!$dbContext->execute($valueArr)) {
            throw new Exception($this->conn->errorInfo()[2]);
        }

        $dbContext = null; /* Database Lock Free */

        self::insertAccessLevel($No, $AccessLevel);

        $this->log->set_log_device('card', 'insert', '0', '', OPENAPI_ID);
        parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("No"=>$No));
    }

    /** ==================================================================
     *  Card Update
     */
    public function update() {
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No             = $PutParams['No'];
            $UserNo         = $PutParams['UserNo'];
            $CardNo         = $PutParams['CardNo'];
            $CardFormatNo   = $PutParams['CardFormatNo'];
            $CardStatus     = $PutParams['CardStatus']; /* Default */
            $CardType       = $PutParams['CardType'];   /* Default */
            $KeyNumber      = $PutParams['KeyNumber'];
            $NeverExpire    = $PutParams['NeverExpire']; 
            $Reason         = strip_tags($PutParams['Reason']);
            $ActDate        = self::to_timestamp($PutParams['ActDate']);
            $ExpDate        = self::to_timestamp($PutParams['ExpDate'], 23, 59, 59);
            $SelectType     = $PutParams['SelectType'];
            $AccessLevel	= array_filter(explode(',',$PutParams['AccessLevel']));
            
            if(empty($No) || !is_numeric($No) || strlen($No) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }
            /* User No Check Valid */
            if(empty($UserNo) || !is_numeric($UserNo)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }
    
            /* Card Format Check Valid */
            $cardFormat = $this->checkCardFormat($CardFormatNo);
    
            if($CardFormatNo == 0 || is_bool($cardFormat)) {            
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0014);
                exit;
            }
    
            $maxCardNumberLength = pow(2, $cardFormat) - 1;
    
             /* Card number Check Valid */
            if(!empty($CardNo)) {
                if(empty($CardNo) || ((int)$CardNo > (int)$maxCardNumberLength) || !is_numeric($CardNo)) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0015);
                    exit;            
                }
            }
        
            /* Access Level List Check Valid */
            if(count($AccessLevel) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0016);
                exit;                        
            }
    
            if(count($AccessLevel) > EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][3]) {
                parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_ACCESS_LEVEL);
                exit;  
            }
    
            /* Limited Card Counter */
            if(self::checkMaxLimitedCard()) {            
                parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_CARD);
                exit;            
            }
    
            /* Max User Card Allocation Counter */
            if(self::checkMaxUserCardAllocation($UserNo)) {
                parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_USERCARD_ALLOCATION);
                exit;
            }
        
            /* NeverExpire가 0 이면 ActDate와 ExpDate 기준으로 처리 (Card.php Logic 기준)
            *  CardType:5(One time) 이면 
            */
            if($CardStatus === '0' &&  $CardType === '5') {
                $ActDate = ''; $ExpDate=''; $NeverExpire='';
            } 
            else {
                if( empty($ActDate) && $NeverExpire == 0 ) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0017);
                    exit;
                }
    
                if( $NeverExpire == 0 ) {
                    if( !empty($ExpDate) && $ActDate > $ExpDate ) {
                        parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0017);
                        exit;
                    }
                } else {
                    $ActDate	= '';
                    $ExpDate	= '';
                }
            }

            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(CardNo) FROM Card WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchAll();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 
                        
            error_log('ActDate:'.$ActDate.", ExpDate:".$ExpDate);

            if(!empty($CardNo)) {
                $valueArr = array($CardNo, $CardFormatNo, $CardStatus,
                                   $KeyNumber, $SelectType, $NeverExpire, $Reason, $ActDate, $ExpDate,
                                   $UserNo, $CardType, parent::getAccountSite(), $No );
        
                $dbContext = $this->conn->prepare("UPDATE Card SET CardNo=?, CardFormatNo=?, ".
                                           " CardStatus=?, KeyNumber=?, SelectType=?, NeverExpire=?, Reason=?, ".
                                           " ActDate=?, ExpDate=?, UserNo=?, CardType=? WHERE Site=? AND No=?");
            }
            else
            {
                $valueArr = array($CardFormatNo, $CardStatus, $KeyNumber, $SelectType, $NeverExpire, $Reason, $ActDate, $ExpDate, $UserNo, $CardType, parent::getAccountSite(), $No );
        
                $dbContext = $this->conn->prepare("UPDATE Card SET CardFormatNo=?, ".
                                           " CardStatus=?, KeyNumber=?, SelectType=?, NeverExpire=?, Reason=?, ".
                                           " ActDate=?, ExpDate=?, UserNo=?, CardType=? WHERE Site=? AND No=?");
            }
    
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

            /* 기존 Access Level 삭제 */
            self::deleteAccessLevel($No);
            /* 새로운 Access Level로 재입력 */
            self::insertAccessLevel($No, $AccessLevel);            

            $this->log->set_log_device('card', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);        
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch

    }

    /** ==================================================================
     *  Card 관련 정보 삭제
     */
    public function delete() {
        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        /* User No Check Valid */
        if(empty($No) || !is_numeric($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(CardNo) FROM Card WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 

            /* 기존 Access Level 삭제 */
            self::deleteAccessLevel($No);

            $dbContext = $this->conn->prepare("DELETE FROM Card WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = null; /* Database Lock Free */

            $this->log->set_log_device('card', 'delete', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch

    }
    /** ==================================================================
     *  Card Access Level을 삭제 시 사용 (Update 시에도 사용 - 기존 데이타 삭제)
     */
    private function deleteAccessLevel($CardNo) {
        try {
            $dbContext = $this->conn->prepare("DELETE FROM CardAccessLevel WHERE Site=? AND CardNo=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $CardNo));
            $dbContext = null; /* Database Lock Free */
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                
    }

    /** ==================================================================
     * Access Level List 
     * INSERT, UPDATE 공용 으로 사용.
     */
    private function insertAccessLevel($CardNo, $AccessLevelList) {

        try {
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM CardAccessLevel WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                            
            $dbContext->execute(array(parent::getAccountSite()));
            $No = $dbContext->fetchColumn(); 
            $dbContext = null; /* Database Lock Free */

            $dbContext = $this->conn->prepare("INSERT INTO CardAccessLevel (Site,No,CardNo, AccessLevelNo) VALUES (?,?,?,?)");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            foreach( $AccessLevelList as $value ) {                
                $valueArr = array(parent::getAccountSite(), $No, $CardNo, $value);                        
                if(!$dbContext->execute($valueArr)) {
                    throw new Exception($this->conn->errorInfo()[2]);
                }

                $No = $No+1;
            }

            $dbContext = null; /* Database Lock Free */

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
    }
    /** ==================================================================
     *  문자열 date를 timestamp format으로 변환
     */
    private function to_timestamp($date="", $hour=0, $min=0, $sec=0) {
        if(strlen($date) == 0 || empty($date)) return "";
        
      	$year   = substr($date, 0, 4);
        $month  = substr($date, 4, 2);
        $day    = substr($date, 6, 2);            
        return mktime($hour, $min, $sec, $month, $day, $year);
    }

    /** ==================================================================
     *  요청받은 Card Format No가 존재유무 확인 
     *  TRUE : 해당 Card Format No가 존재 함
     *  FALSE : 존재하지 않는 Card Format No
     */
    private function checkCardFormat($no) {
        try {
            $dbContext = $this->conn->prepare("SELECT IFNULL(CardNumberLength,'') CardNumberLength FROM CardFormat Where No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            if(!$dbContext->execute(array($no))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            //$lists = $dbContext->fetchColumn(PDO::FETCH_ASSOC);  
            $lists = $dbContext->fetchColumn();  
            $dbContext = null; /* Database Lock Free */

            //if(strlen($lists['CardNumberLength']) > 0) return $lists['CardNumberLength'];
            if(strlen($lists) > 0) return $lists;
             else FALSE;

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /** ==================================================================
     * License에 따른 최대 Card Counter 확인
     * TRUE : 최대  Card Counter 초과
     * FALSE : 최대 Card Counter 이하
     */
    private function checkMaxLimitedCard() {
        $maxUser = EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][4];
        if($this->util->GetRecordCount('Card') >= $maxUser) return TRUE;
        else FALSE;   
     }

    /** ==================================================================
     *  사용자 단위 최대 등록 가능 카드 갯수 확인
     */
    private function checkMaxUserCardAllocation($UserNo) {
        $maxCard = EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][5];
        if($this->util->GetRecordCount('Card', " WHERE UserNo='".$UserNo."'") >= $maxCard) return TRUE;
        else FALSE; 
    }

    /** ==================================================================
     *  중복카드 사용 여부 확인
     */
    private function checkDuplicationCard($cardNo, $cardFormat) {
        if($this->util->GetRecordCount('Card', " WHERE CardNo='".$cardNo."' AND CardFormatNo='".$cardFormat."'") > 0) return TRUE;
        else FALSE; 
    }

    private function  to_list_AccessLevel($No) {
        $list = $this->conn->prepare("SELECT AccessLevelNo FROM CardAccessLevel WHERE Site=? AND CardNo=?");

        //$list   = $this->conn->prepare("SELECT ReaderNo FROM AccessLevelReader WHERE Site=? AND AccessLevelNo=?");

        $list->execute(array(parent::getAccountSite(), $No));
        $list = $list->fetchAll(PDO::FETCH_COLUMN);
        $AlList = implode(", ", $list);
        
        return $AlList;
    }

}
?>