<?php
/***********************************************************************************
 * FILE NAME : CardFormat.php                                          DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.05.23 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.05.23  CJMOON     Init 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : CardFormat API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";

class CardFormat extends OpenApiBaseImpl
{
    /* ================================================================== */    
    /* Card Format List 정보 */
    public function clist() {

        $page   = $this->input->get('page', 1);   /* 요청 할 Page 번호 */        

        try {
            $dbContext = $this->conn->prepare("SELECT COUNT(No) FROM CardFormat");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
             }

            $dbContext->execute();
            $totalUser = (int)$dbContext->fetchColumn();        
            $dbContext = null; /* Database Lock Free */
            
            $dbContext = $this->conn->prepare("SELECT No, Name, Mean, TotalBitLength, FacilityCode, FacilityBitLength, CardNumberLength, ".
                                              " CardNumberStartBit, isDefault from CardFormat ORDER BY NO DESC LIMIT ?,?");

            if(is_bool($dbContext)) {
              throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getCurrentPaging($page), MAX_LIST_ROW));  /* Reocrd를 20개 단위로 사용 */
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

            if(count($dbContext) > 0) {
                $vars = array( "total"=>$totalUser, "totalPage"=>parent::getPaging($totalUser), 'rows' => $dbContext);
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($vars));
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }   

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /* ================================================================== */  
    /** Card Format Information Insert
     * 
     */    
    public function insert() {
        $Name			     = strip_tags(trim($this->input->post('Name')));
		$Mean			     = strip_tags($this->input->post('Mean'));
		$TotalBitLength		 = $this->input->post('TotalBitLength');
		$FacilityCode		 = $this->input->post('FacilityCode');
		$FacilityBitLength	 = $this->input->post('FacilityBitLength');
		$FacilityStartBit	 = $this->input->post('FacilityStartBit');
		$CardNumberLength	 = $this->input->post('CardNumberLength');		
        $CardNumberStartBit	 = $this->input->post('CardNumberStartBit');
        
        if(empty($Name) || 
           $TotalBitLength === '' || !is_numeric($TotalBitLength) ||
           $FacilityCode === ''   || !is_numeric($FacilityCode)   ||
           $FacilityBitLength === '' || !is_numeric($FacilityBitLength)||
           $FacilityStartBit === ''  || !is_numeric($FacilityStartBit) ||
           $CardNumberLength === ''  || !is_numeric($CardNumberLength) ||
           $CardNumberStartBit === ''|| !is_numeric($CardNumberStartBit)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        if(($TotalBitLength > 64) || 
           ($FacilityStartBit > $TotalBitLength) ||
           ($FacilityBitLength > ($TotalBitLength - $FacilityStartBit))) {
             parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
             exit;            
        }

        if( $CardNumberStartBit < $FacilityStartBit ) {
            if($CardNumberLength > ($FacilityStartBit - $CardNumberStartBit)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }
        }
        else if($CardNumberStartBit >= ($FacilityStartBit + $FacilityBitLength)) {
            if($CardNumberLength > ($TotalBitLength - $CardNumberStartBit + 1)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                
            }
        } else {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;                            
        }

        $max_facility_code	= pow(2, $FacilityBitLength) - 1;

        if(((int)$FacilityCode < 0) || ((int)$FacilityCode > $max_facility_code)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;                            
        }
        
        /* Key Create */
        $dbContext = $this->conn->prepare("SELECT (IFNULL(MAX(No), 0)+1) No FROM CardFormat");
        if(is_bool($dbContext)) {
           throw new Exception($this->conn->errorInfo()[2]);
        }

        $dbContext->execute();
        $No = $dbContext->fetchColumn();
        $dbContext = null; /* Database Lock Free */

        $valueArr = array($No, $Name,$Mean,$FacilityCode,$TotalBitLength,$FacilityBitLength,$FacilityStartBit,$CardNumberLength,$CardNumberStartBit);
        $dbContext = $this->conn->prepare("INSERT INTO CardFormat (No,Name,Mean,FacilityCode,TotalBitLength,".
                                          "FacilityBitLength,FacilityStartBit,CardNumberLength,CardNumberStartBit)".
                                          "VALUES (?,?,?,?,?,?,?,?,?)");

        if(is_bool($dbContext)) {
            throw new Exception($this->conn->errorInfo()[2]);
        }

        if(!$dbContext->execute($valueArr)) {
            throw new Exception($this->conn->errorInfo()[2]);
        }

        $dbContext = null; /* Database Lock Free */

        $this->log->set_log_device('card_format', 'insert', '0', '', OPENAPI_ID);
        parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("No"=>$No));
    }

    /** ==================================================================
     *  Card Format Update
     */    
    public function update() {
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No			         = $PutParams['No'];
            $Name			     = $PutParams['Name'];
            $Mean			     = $PutParams['Mean'];
            $TotalBitLength		 = $PutParams['TotalBitLength'];
            $FacilityCode		 = $PutParams['FacilityCode'];
            $FacilityBitLength	 = $PutParams['FacilityBitLength'];
            $FacilityStartBit	 = $PutParams['FacilityStartBit'];
            $CardNumberLength	 = $PutParams['CardNumberLength'];	
            $CardNumberStartBit	 = $PutParams['CardNumberStartBit'];
            
            if(empty($Name) || 
               $TotalBitLength === '' || !is_numeric($TotalBitLength) ||
               $FacilityCode === ''   || !is_numeric($FacilityCode)   ||
               $FacilityBitLength === '' || !is_numeric($FacilityBitLength)||
               $FacilityStartBit === ''  || !is_numeric($FacilityStartBit) ||
               $CardNumberLength === ''  || !is_numeric($CardNumberLength) ||
               $CardNumberStartBit === ''|| !is_numeric($CardNumberStartBit)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }
    
            if(($TotalBitLength > 64) || 
               ($FacilityStartBit > $TotalBitLength) ||
               ($FacilityBitLength > ($TotalBitLength - $FacilityStartBit))) {
                 parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                 exit;            
            }
    
            if( $CardNumberStartBit < $FacilityStartBit ) {
                if($CardNumberLength > ($FacilityStartBit - $CardNumberStartBit)) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                    exit;
                }
            }
            else if($CardNumberStartBit >= ($FacilityStartBit + $FacilityBitLength)) {
                if($CardNumberLength > ($TotalBitLength - $CardNumberStartBit + 1)) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                    exit;                
                }
            } else {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                            
            }
    
            $max_facility_code	= pow(2, $FacilityBitLength) - 1;
    
            if(((int)$FacilityCode < 0) || ((int)$FacilityCode > $max_facility_code)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                            
            }

            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM CardFormat WHERE No=?");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            }

            $valueArr = array($Name, $Mean, $FacilityCode,$TotalBitLength,$FacilityBitLength,$FacilityStartBit,$CardNumberLength, $CardNumberStartBit, $No);


            $dbContext = $this->conn->prepare("UPDATE CardFormat SET Name=?, Mean=?, FacilityCode=?, TotalBitLength=?, " .
                                              "FacilityBitLength=?, FacilityStartBit=?, CardNumberLength=?, CardNumberStartBit=? " .
                                              " WHERE No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);                
            }

            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext = null; /* Database Lock Free */

            $this->log->set_log_device('card_format', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /** ==================================================================
     *  Card Format Delete
     */    
    public function delete() {
        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        
        /* Card Format No Check Valid */
        if(empty($No) || !is_numeric($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM CardFormat WHERE No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 

            $dbContext = $this->conn->prepare("DELETE FROM CardFormat WHERE No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            if(!$dbContext->execute(array($No))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext = null; /* Database Lock Free */
            $this->log->set_log_device('card_format', 'delete', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);            

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /** ==================================================================
     *  Default Card Format API
     */     
    public function defaultFormat() {
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);
        
        $No = $PutParams['No'];
        
        /* Card Format No Check Valid */
        if(empty($No) || !is_numeric($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {

            $dbContext = $this->conn->prepare("UPDATE CardFormat SET IsDefault=0");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            if(!$dbContext->execute()) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = $this->conn->prepare("UPDATE CardFormat SET IsDefault=1 WHERE No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            if(!$dbContext->execute(array($No))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext = null; /* Database Lock Free */
            $this->log->set_log_device('card_format', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);            

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
    }
}

?>