<?php
/***********************************************************************************
 * FILE NAME : Client.php                                        DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.01.03 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Client API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class Client extends OpenApiBaseImpl
{
    /* ================================================================== */    
    /* Access Level Lsit 정보 */
    public function clist()
    {
        try {	        
            $list = $this->conn->prepare("SELECT No, Name, Mac, IP, Type, Version, PortCount, Alive FROM Host WHERE No != '1' ORDER BY No DESC");
            if(is_bool($list)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $list->execute();
            $list   = $list->fetchAll(PDO::FETCH_ASSOC);
            if(count($list) > 0) {
                foreach( $list as $key=>$val ) {

                    $list[$key]['str'] =  EnumTable::$attrModelType[$val['Type']][$val['PortCount']];

                    $list[$key]['AliveName'] = $val['Alive']==="0"?"Off":"On";
                    $list[$key]['TypeName'] = EnumTable::$attrModel_NT[$val['Type']];
                    
                    if(substr($val['Version'], 0, 1) == '5') $list[$key]['Modelno'] = EnumTable::$neptuneProductModelNo_NT[5]; /* Neptume H5 Model No */
                    else if(substr($val['Version'], 0, 1) == '3') $list[$key]['Modelno'] = EnumTable::$neptuneProductModelNo_NT[3]; /* Neptume H3 Model No */
                    else if(substr($val['Version'], 0, 1) == '9') $list[$key]['Modelno'] = EnumTable::$neptuneProductModelNo_NT[9]; /* Neptume Giant No */
                    else if(substr($val['Version'], 0, 1) == '1') $list[$key]['Modelno'] = EnumTable::$neptuneProductModelNo_NT[1]; /* Neptume Giant No */
                    else $list[$key]['Modelno'] = EnumTable::$neptuneProductModelNo_NT[0]; /* E3-SPIDER */        
                }
                
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($list));             
            }
            else {
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);             
            }
		} // try
		catch (PDOException $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		catch(Exception $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch                    
    }

    /* ==================================================================*/
    /* Client 연결*/
    /* OpenAPI에서는 SITE개념이 없이 모든 자원에 대한 제어권을 사용한다.*/
    public function connect()
    {
        $No     = $this->input->get('clientNo');
        
        if(empty($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE, 
            ConstOpenapiErrorCode::getErrorMessage(ConstOpenapiErrorCode::ERR_INPUT_VALUE));
            exit;
        }

        try {	 
            $dbContext  = $this->conn->prepare("SELECT Alive FROM Host WHERE No != '1' and No = ? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $dbContext  = $dbContext->fetchColumn();
            
            if(count($dbContext) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }

            if($dbContext == 0) { /* Disconnect */
                parent::toError(ConstOpenapiErrorCode::ERR_CLIENT_NOT_ALIVE);
                exit;
            }

            //server_client.php 예제
            exec(SPIDER_COMM." clntdb enable ".$No." ".parent::getAccountSite()." ".$_SERVER['REMOTE_ADDR']." ".OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch               
    }

    /* ================================================================== */    
    /* Client 연결해제 */    
    public function disconnect()
    {
        $No     = $this->input->get('clientNo');
        
        if(empty($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        try {	 
            $dbContext  = $this->conn->prepare("SELECT Alive FROM Host WHERE No != '1' and No = ? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $dbContext  = $dbContext->fetchColumn();
            
            if(count($dbContext) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }

            if($dbContext == 0) { /* Disconnect */
                parent::toError(ConstOpenapiErrorCode::ERR_CLIENT_NOT_ALIVE);
                exit;
            }

            //server_client.php 예제
            exec(SPIDER_COMM." clntdb disable ".$No." ".parent::getAccountSite()." ".$_SERVER['REMOTE_ADDR']." ".OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch  
    }

    /* ================================================================== */    
    /* Client Delete */        
    public function delete()
    {
        $No     = $this->input->get('clientNo');
        
        if(empty($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        try {	 
            $dbContext  = $this->conn->prepare("SELECT Alive FROM Host WHERE No != '1' and No = ? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $dbContext  = $dbContext->fetchColumn();
            
            if(count($dbContext) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }

            //server_client.php 예제
            exec(SPIDER_COMM." clntdb disable ".$No." ".parent::getAccountSite()." ".$_SERVER['REMOTE_ADDR']." ".OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch          
    }

    /* ================================================================== */    
    /* Client Reboot */            
    public function reboot()
    {
        $No     = $this->input->get('clientNo');
        
        if(empty($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        try {	 
            $dbContext  = $this->conn->prepare("SELECT Alive FROM Host WHERE No != '1' and No = ? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $dbContext  = $dbContext->fetchColumn();
            
            if(count($dbContext) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }

            //server_client.php 예제
            exec(SPIDER_COMM." clientreboot ".$No);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                  
    }

    /* ================================================================== */    
    /* Client Package Upgrade */                
    public function upgrade()
    {
        $No     = $this->input->get('clientNo');
        
        if(empty($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        try {	 
            $dbContext  = $this->conn->prepare("SELECT Alive FROM Host WHERE No != '1' and No = ? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array($No));
            $dbContext  = $dbContext->fetchColumn();
            
            if(count($dbContext) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }

            if($dbContext == 0) /* Disconnect */
            {
                parent::toError(ConstOpenapiErrorCode::ERR_CLIENT_NOT_ALIVE);
                exit;
            }

            //server_client.php 예제, 현재 Type 변수는 사용되지 않음.
            //$rt = exec(SPIDER_COMM." clientupdate ".$No." ".$Type);
            $rt = exec(SPIDER_COMM." clientupdate ".$No." "."0");

            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array());
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch          
    }

    /* ================================================================== */    
    /* Client Database Sync */
    public function sync()
    {
        try {	         
            exec(SPIDER_COMM." send db");
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array());
        }
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                      
    }
}

?>