<?php
/***********************************************************************************
 * FILE NAME : Control.php                         DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2018.12.26 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Log query
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";

class Control extends OpenApiBaseImpl
{   
    /* ================================================================== */    
    /* Door Manual Open/Close Command */    
    public function door()
    {
        $sel_door   = $this->input->get('doorNo');
		$lock_mode  = $this->input->get('lockMode');

        if(empty($sel_door) || empty($lock_mode)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        /* Door 제어를 위한 명령 */
        if(strtolower($lock_mode) == "m") $ActionMode = "m_unlock";
        else if(strtolower($lock_mode) == "u") $ActionMode = "unlocked";
        else if(strtolower($lock_mode) == "l") $ActionMode = "locked";
        else {
            parent::toError(ConstOpenapiErrorCode::ERR_DOOR_CONTROL_NOT_ACTION_MODE_MODE);
            exit;
        }
    
        try {           
            $dbContext = $this->conn->prepare("SELECT Name, LockMode, Port FROM Door WHERE Site=? AND No=? AND Disable=0");
            $dbContext->execute(array(parent::getAccountSite(),$sel_door) );
            $dbContext = $dbContext->fetch(PDO::FETCH_ASSOC);

            if(count($dbContext) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;
            }
    
            if( $dbContext['LockMode'] > '0' && $dbContext['LockMode'] < '4' ) {
                parent::toError(ConstOpenapiErrorCode::ERR_DOOR_CONTROL_NOT_NORMAL);
                exit;
            }
            else if($dbContext['LockMode'] == '4') {            
                parent::toError(ConstOpenapiErrorCode::ERR_DOOR_CONTROL_MANTRAP);
                exit;
            }    
            
            $this->log->set_log_device('door', $ActionMode, '0', $dbContext['Port'], $dbContext['Name'].'('.OPENAPI_ID.')');
            exec(SPIDER_COMM." door $sel_door $ActionMode forced");            
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch            
    }

    /* ================================================================== */    
    /* Aux Output Manual Open/Close Command */
    public function auxout()
    {
        $sel_auxoutput  = $this->input->get('auxOutNo');
        $onoff          = $this->input->get('actionMode');
        $elevatorMode   = $this->input->get('elevatorMode');
        
        if(empty($sel_auxoutput) || empty($onoff)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        if(strtolower($onoff) === "on") $ActionMode = "on";
        else if(strtolower($onoff) === "off") $ActionMode = "off";
        else {
            parent::toError(ConstOpenapiErrorCode::ERR_DOOR_CONTROL_NOT_ACTION_MODE_MODE);
            exit;
        }
        
        /* TODO 엘레베이터 처리는 어떻게 해야 할지 고민 필요 */
        try {   
            if(strtolower($elevatorMode) == 'on') {
                $dbContext   = $this->conn->prepare("SELECT Name, Port FROM ElevatorRelay WHERE Site=? AND No = ? AND Disable=0");
               
                if(!$dbContext->execute(array(parent::getAccountSite(), $sel_auxoutput))) {
                    throw new Exception($this->conn->errorInfo()[2]);
                }

                $dbContext   = $dbContext->fetch(PDO::FETCH_ASSOC);
                
                if(is_bool($dbContext) || count($dbContext) == 0) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                    exit;
                }
                
                $this->log->set_log_device('elevator', $ActionMode, '0', $dbContext['Port'], $dbContext['Name'].'('.OPENAPI_ID.')');            
                exec(SPIDER_COMM." elevatorout $sel_auxoutput $ActionMode");
            } 
            else {
                $dbContext   = $this->conn->prepare("SELECT Name, Port FROM AuxOutput WHERE Site=? AND No = ? AND Disable=0");
                $dbContext->execute( array(parent::getAccountSite(), $sel_auxoutput) );
                $dbContext   = $dbContext->fetch(PDO::FETCH_ASSOC);

                if(is_bool($dbContext) || count($dbContext) == 0) {
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                    exit;
                }

                $this->log->set_log_device('aoutput', $ActionMode, '0', $dbContext['Port'], $dbContext['Name'].'('.OPENAPI_ID.')');
                shell_exec(SPIDER_COMM." auxout $sel_auxoutput $ActionMode");
            } // if end
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch     
        
        parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
    }
}
?>