<?php
/***********************************************************************************
 * FILE NAME : Door.php                                          DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.05.23 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.01.03  CJMOON     Init 
 *        2    0.0.2       2019.11.26  CJMOON     add state
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Door Setting API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";

class Door extends OpenApiBaseImpl
{
   
    /* ================================================================== */    
    /* Door And Door Contact State */
    public function state() {
        try {
            /* 
               Type : 1: Door Contact, 2: Aux Output, 3: Door Lock, 4: Elevator Out ( Aux Output ) 
            */
            $dbContext = $this->conn->prepare("select a.No, a.HostNo, a.Name, a.Mean, a.DoorContactState, c.port, b.type, b.RelayOn " .
                                              " from (select No,HostNo, Name, Mean, DoorContactState from door) a, " .
                                              "      (select * from RelayStatus where Type IN(1, 3)) b, (select No, Port from doorcontact) c " .
                                              " where a.No = b.no and b.no = c.no order by a.HostNo, a.No");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
             }
            
            $dbContext->execute(); 
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

            if(count($dbContext) > 0) {
                foreach( $dbContext as $key=>$val ) {
                    if($val['Type']==1) {
                        $dbContext[$key]['State'] = $val['RelayOn'];
                        $dbContext[$key]['TypeStr'] = "Contact";

                        if($val['RelayOn']==1) $dbContext[$key]['StateStr'] = "Door Open";
                        else if($val['RelayOn']==2) $dbContext[$key]['StateStr'] = "Door Close";
                        else if($val['RelayOn']==2) $dbContext[$key]['StateStr'] = "Forced Open";                       
                    }
                    else if($val['Type']==3) {
                        $dbContext[$key]['TypeStr'] = "Lock";

                        if($val['DoorContactState'] == 0)  { //De-Enegized
                            $dbContext[$key]['State'] = $val['RelayOn'];
                            //$dbContext[$key]['StateStr']= $val['RelayOn']==1?"Door Unlock":"Door Lock";
                        }
                        else if($val['DoorContactState'] == 1) {
                            $dbContext[$key]['State'] = $val['RelayOn']==1?"2":"1";
                            //$dbContext[$key]['StateStr']= $val['RelayOn']==1?"Door Lock":"Door UnLock";
                        }                        

                        $dbContext[$key]['StateStr']= $dbContext[$key]['State']==2?"Door Lock":"Door UnLock";
                    }

                    unset($dbContext[$key]['No']);
                    unset($dbContext[$key]['RelayOn']);
                    unset($dbContext[$key]['DoorContactState']);
                }
                
               parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($dbContext));
            } 
            else {
               parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }   
            
            } // try
            catch (PDOException $e) {
                parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
                exit;
            } // catch
            catch(Exception $e) {
                parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
                exit;
            } // catch   


    }

    /* ================================================================== */    
    /* Door Lsit 정보 */    
    public function clist() {
        $page   = $this->input->get('page', 1);   /* 요청 할 Page 번호 */

        try {
            $dbContext = $this->conn->prepare("SELECT COUNT(No) FROM Door WHERE Site=? AND Disable='0'");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
             }

            $dbContext->execute(array(parent::getAccountSite()));
            $totalDoor = (int)$dbContext->fetchColumn();     

            $dbContext = $this->conn->prepare("SELECT a.No No, a.Name DoorName, b.name ClientName, b.ip ClientIP, a.Mean ClientDoor, a.Floor Floor, a.LockMode LockMode, ".
            "a.LockTime LockTime, a.AdaOpenTime AdaOpenTime, a.ForcedEnable ForcedEnable, a.HeldEnable HeldEnable, a.HeldOpenTime HeldOpenTime ".
            "from Door a, Host b where a.hostno = b.no and a.disable = '0' and a.site=? order by a.no  DESC LIMIT ?,? ");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
              }
  
              $dbContext->execute(array(parent::getAccountSite(), parent::getCurrentPaging($page), MAX_LIST_ROW));  /* Reocrd를 20개 단위로 사용 */
              $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

              if(count($dbContext) > 0) {
                $vars = array( "total"=>$totalDoor, "totalPage"=>parent::getPaging($totalDoor), 'rows' => $dbContext);
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($vars));
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }   

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch       
    }
}
?>