<?php
/***********************************************************************************
 * FILE NAME : DoorGroup.php                                     DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.06.14 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.06.14  CJMOON     Init 
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Access Level Group API
 *  
 ***********************************************************************************/
include_once "./common/OpenApiBaseImpl.php";
include_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';
require_once APP_DIR.'/libraries/ConstTable.php';

class DoorGroup extends OpenApiBaseImpl
{
    
    /* ================================================================== */    
    /* AccessLevelGroup Process */
    public function clist() 
    {
        try {	        
            $page   = $this->input->get('page', 1);   /* 요청 할 Page 번호 */

            $dbContext = $this->conn->prepare("SELECT COUNT(No) FROM GroupTable WHERE Site=? AND GroupKind=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
             }

            $dbContext->execute(array(parent::getAccountSite(), ConstTable::GROUP_DOOR));
            $totalDoor = (int)$dbContext->fetchColumn();

            $dbContext = $this->conn->prepare("SELECT No, Name From GroupTable WHERE Site=? AND GroupKind=? ORDER BY No DESC LIMIT ?, ?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
  
            $dbContext->execute(array(parent::getAccountSite(), ConstTable::GROUP_DOOR, parent::getCurrentPaging($page), MAX_LIST_ROW));  /* Reocrd를 20개 단위로 사용 */
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

            if(count($dbContext) > 0) {
                $vars = array( "total"=>$totalDoor, "totalPage"=>parent::getPaging($totalDoor), 'rows' => $dbContext);
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($vars)); 
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }                        
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch             
    }

    /* ================================================================== */    
    /* Door Group 추가
     */
    public function insert()
    {
        $Name       = strip_tags(trim($this->input->post('Name')));
        $DoorList   = array_filter(explode(',',$this->input->post('DoorList')));
        
        try {        
            if(empty($Name) || $Name === '' || 
                count($DoorList) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;             
            }

            /* User Key Create */
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM GroupTable WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                
            if(!$dbContext->execute(array(parent::getAccountSite()))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $No = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */
            $valueArr  = array(parent::getAccountSite(), $No, $Name, ConstTable::GROUP_DOOR);
            $dbContext = $this->conn->prepare("INSERT INTO GroupTable (Site, No, Name, GroupKind) VALUES (?,?,?,?)");
                                    
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                            
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            } 

            $dbContext = null; /* Database Lock Free */

            self::insertDoorGroup($No, $DoorList);

            $this->log->set_log_device('groupdoor', 'insert', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("No"=>$No));     
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch  
                
    }    

   /** ==================================================================
     *  Access Level Group Update
     */    
    public function update() {
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No         = $PutParams['No'];
            $Name       = strip_tags(trim($PutParams['Name']));
            $DoorList   = array_filter(explode(',',$PutParams['DoorList']));
            
            if(empty($No) || !is_numeric($No) || $No === '') {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }

            if(empty($Name) || $Name === '' ||
               count($DoorList) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;             
            }

            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM GroupTable WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchAll();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            }

            $valueArr = array($Name, parent::getAccountSite(), $No);
            $dbContext = $this->conn->prepare("UPDATE GroupTable SET Name=? WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

            /* 기존 Access Level Group (Sub) 삭제 */
            self::deleteDoorGroup($No);
            /* 새로운 Access Level Reader로 재입력 */
            self::insertDoorGroup($No, $DoorList);  

            $this->log->set_log_device('groupdoor', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch       

    }    

    /** ==================================================================
     *  Access Level Group Delete
     */        
    public function delete() {
        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        /* Access Level Group Check Valid */
        if(empty($No) || !is_numeric($No) || $No === '') {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM GroupTable WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 

            /* 기존 Access Level List 삭제 */
            self::deleteDoorGroup($No);

            $dbContext = $this->conn->prepare("DELETE FROM GroupTable WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = null; /* Database Lock Free */

            $this->log->set_log_device('groupdoor', 'delete', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch

    }    

    /** ==================================================================
     * Access Level Group List 
     * INSERT, UPDATE 공용 으로 사용.
     */
    private function insertDoorGroup($GroupNo, $DoorlList) {

        try {
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM GroupElement WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                            
            $dbContext->execute(array(parent::getAccountSite()));
            $No = $dbContext->fetchColumn(); 
            $dbContext = null; /* Database Lock Free */

            $dbContext = $this->conn->prepare("INSERT INTO GroupElement (Site,No,GroupNo,GroupKind,ElementNo,Disable) VALUES (?,?,?,?,?,?)");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            foreach( $DoorlList as $value ) {                
                $valueArr = array(parent::getAccountSite(), $No, $GroupNo, ConstTable::GROUP_DOOR, $value, "0");                        
                if(!$dbContext->execute($valueArr)) {
                    throw new Exception($this->conn->errorInfo()[2]);
                }

                $No = $No+1;
            }

            $dbContext = null; /* Database Lock Free */

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
    }

    /** ==================================================================
     *  Door Group을 삭제 시 사용 (Update 시에도 사용 - 기존 데이타 삭제)
     */
    private function deleteDoorGroup($GroupNo) {
        try {
            $dbContext = $this->conn->prepare("DELETE FROM GroupElement WHERE Site=? AND GroupNo=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $GroupNo));
            $dbContext = null; /* Database Lock Free */
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                
    }    
}
?>