<?php
/***********************************************************************************
 * FILE NAME : Login.php                                         DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.12.04 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.12.04  CJMOON     Init 
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : 
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";


class Login extends OpenApiBaseImpl
{
    /* ================================================================== */    
    /* Login Request */
    public function clist() 
    {
        $loginid   = $this->input->get('loginid');
        $loginpw   = $this->input->get('loginpw');
        
        try {	
            $ctrl   = $this->conn->query("SELECT * FROM Controller WHERE ID = ? AND PASSWORD = ? AND No = 1");
            if(is_bool($ctrl)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $ctrl->execute(array($loginid, hash('sha256', $loginpw, false)));
            $ctrl = $ctrl->fetch(PDO::FETCH_ASSOC);

            if(is_bool($ctrl)) {
                parent::toJson(ConstOpenapiErrorCode::ERR_LOGIN_FAIL);
                exit;
            }
                        
            session_start();
            $authKeyValue = hash_hmac("sha256", $_SERVER['REMOTE_ADDR'], "sicunet2019!");
            $_SESSION['openapi_key'] = $authKeyValue;
            session_write_close();

            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array(array("session"=>$authKeyValue)));    
        }
        catch (PDOException $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		catch(Exception $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch                           
    }

    /* ================================================================== */    
    /* Session Expired */    
    public function Expired()
    {
        $authKey = hash_hmac("sha256", time(), parent::getHashKey());

        try {
            $dbContext = $this->conn->prepare("UPDATE OPENAPI SET AuthKey=?");
    
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                
            if(!$dbContext->execute(array($authKey))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }        
            
            $dbContext = null; /* Database Lock Free */
            self::setAuthKey($authKey);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array(array("session"=>$authKey, "env"=>$_ENV["OPENAPI_DATA"])));    
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }
}
 ?>