<?php
/***********************************************************************************
 * FILE NAME : Schedule.php                                     DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.05.27 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.05.22  CJMOON     Init 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Schedule Setting API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class Schedule extends OpenApiBaseImpl
{

    /** ==================================================================
     *  현재의 Schedule List정보 반환
     */
    public function clist() {
        try {
            $dbContext = $this->conn->prepare("SELECT No, Name, Mean, sun_st ||'~' || sun_ed SunTime, mon_st ||'~' || " .
                                              "mon_ed MonTime, tue_st ||'~' || tue_ed TueTime, wed_st ||'~' || wed_ed WedTime,".
                                              "thu_st ||'~' || thu_ed ThuTime,fri_st ||'~' || fri_ed FriTime,".
                                              "sat_st ||'~' || sat_ed SatTime, hol_st ||'~' || hol_ed HolidayTime,".
                                              "sun_reverse SunReverseUse, mon_reverse MonReverseUse,tue_reverse TueReverseUse,".
                                              "wed_reverse WedReverseUse, thu_reverse ThuReverseUse, fri_reverse FriReverseUse,".
                                              "sat_reverse SatReverseUse, hol_reverse HolidayReverseUse, Holiday1, Holiday2, Holiday3, Holiday4 ".
                                              "from Schedule where Site = ?");
    
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
    
            $dbContext->execute(array(parent::getAccountSite()));
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);
    
            if(count($dbContext) > 0) {               
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, $dbContext);
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }   
    
            } // try
            catch (PDOException $e) {
                parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
                exit;
            } // catch
            catch(Exception $e) {
                parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
                exit;
            } // catch
        }

    /* ================================================================== */  
    /** Schedule Insert
     * 
     */
    public function insert() {        

        try {        
            $Name           = strip_tags(trim($this->input->post('Name')));
            $Name           = str_replace("'", "", $Name);
            $Mean           = strip_tags($this->input->post('Mean'));
            $Mean           = str_replace("'", "", $Mean);

            $SunReverseUse            = $this->input->post('SunReverseUse', 0);        
            list($SunStartTime, $SunEndTime)  = self::parseInputTime($this->input->post('SunTime', '0000~0000'));
            $MonReverseUse            = $this->input->post('MonReverseUse', 0);
            list($MonStartTime, $MonEndTime)  = self::parseInputTime($this->input->post('MonTime', '0000~0000'));
            $TueReverseUse            = $this->input->post('TueReverseUse', 0);
            list($TueStartTime, $TueEndTime)  = self::parseInputTime($this->input->post('TueTime', '0000~0000'));
            $WedReverseUse            = $this->input->post('WedReverseUse', 0);
            list($WedStartTime, $WebEndTime)  = self::parseInputTime($this->input->post('WedTime', '0000~0000'));
            $ThuReverseUse            = $this->input->post('ThuReverseUse', 0);
            list($ThuStartTime, $ThuEndTime)  = self::parseInputTime($this->input->post('ThuTime', '0000~0000'));
            $FriReverseUse            = $this->input->post('FriReverseUse', 0);
            list($FriStartTime, $FriEndTime)  = self::parseInputTime($this->input->post('FriTime', '0000~0000'));
            $SatReverseUse            = $this->input->post('SatReverseUse', 0);
            list($SatStartTime, $SatEndTime)  = self::parseInputTime($this->input->post('SatTime', '0000~0000'));
            $HolidayReverseUse            = $this->input->post('HolidayReverseUse', 0);
            list($HolidayStartTime, $HolidayEndTime)  = self::parseInputTime($this->input->post('HolidayTime', '0000~0000'));
            $Holiday1       = $this->input->post('Holiday1', 0);
            $Holiday2       = $this->input->post('Holiday2', 0);
            $Holiday3       = $this->input->post('Holiday3', 0);
            $Holiday4       = $this->input->post('Holiday4', 0);
           
            if(empty($Name) || 
              ($SunStartTime > $SunEndTime) || !is_numeric($SunStartTime) || !is_numeric($SunEndTime) ||
              ($MonStartTime > $MonEndTime) || !is_numeric($MonStartTime) || !is_numeric($MonEndTime) ||
              ($TueStartTime > $TueEndTime) || !is_numeric($TueStartTime) || !is_numeric($TueEndTime) ||
              ($WedStartTime > $WebEndTime) || !is_numeric($WedStartTime) || !is_numeric($WebEndTime) ||
              ($ThuStartTime > $ThuEndTime) || !is_numeric($ThuStartTime) || !is_numeric($ThuEndTime) ||
              ($FriStartTime > $FriEndTime) || !is_numeric($FriStartTime) || !is_numeric($FriEndTime) ||
              ($SatStartTime > $SatEndTime) || !is_numeric($SatStartTime) || !is_numeric($SatEndTime) ||
              ($HolidayStartTime > $HolidayEndTime) || !is_numeric($HolidayStartTime) || !is_numeric($HolidayEndTime)) { 
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }
            
            if((intval($SunStartTime) < 0 ) || (intval($SunStartTime) > 2359) ||
               (intval($SunEndTime) < 0 ) || (intval($SunEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($MonStartTime) < 0 ) || (intval($MonStartTime) > 2359) ||
               (intval($SunEndTime) < 0 ) || (intval($SunEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($TueStartTime) < 0 ) || (intval($TueStartTime) > 2359) ||
               (intval($TueEndTime) < 0 ) || (intval($TueEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($WedStartTime) < 0 ) || (intval($WedStartTime) > 2359) ||
               (intval($WebEndTime) < 0 ) || (intval($WebEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($ThuStartTime) < 0 ) || (intval($ThuStartTime) > 2359) || 
               (intval($ThuEndTime) < 0 ) || (intval($ThuEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($FriStartTime) < 0 ) || (intval($FriStartTime) > 2359) || 
               (intval($FriEndTime) < 0 ) || (intval($FriEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($SatStartTime) < 0 ) || (intval($SatStartTime) > 2359) || 
               (intval($SatEndTime) < 0 ) || (intval($SatEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($HolidayStartTime) < 0 ) || (intval($HolidayStartTime) > 2359) || 
               (intval($HolidayEndTime) < 0 ) || (intval($HolidayEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }            

            /* 라이센스에 따른 최대 사용 가능 갯수 확인 */
            if($this->checkMaxLimitedSchedule()) {            
                parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_SCHEDULE);
                exit;            
            }

            /* User Key Create */
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM Schedule WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                            
            if(!$dbContext->execute(array(parent::getAccountSite()))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $No = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            $valueArr = array(parent::getAccountSite(), $No, $Name, $Mean, $SunReverseUse, $SunStartTime, $SunEndTime,
                              $MonReverseUse, $MonStartTime, $MonEndTime, $TueReverseUse, $TueStartTime, $TueEndTime, 
                              $WedReverseUse, $WedStartTime, $WebEndTime, $ThuReverseUse, $ThuStartTime, $ThuEndTime, 
                              $FriReverseUse, $FriStartTime, $FriEndTime, $SatReverseUse, $SatStartTime, $SatEndTime,
                              $HolidayReverseUse, $HolidayStartTime, $HolidayEndTime, 
                              $Holiday1, $Holiday2, $Holiday3, $Holiday4);


            $dbContext = $this->conn->prepare("INSERT INTO Schedule (Site, No, Name, Mean, sun_reverse, sun_st, sun_ed, ".
                                              "mon_reverse, mon_st, mon_ed, tue_reverse, tue_st, tue_ed, ".
                                              "wed_reverse, wed_st, wed_ed, thu_reverse, thu_st, thu_ed,".
                                              "fri_reverse, fri_st, fri_ed, sat_reverse, sat_st, sat_ed,".
                                              "hol_reverse, hol_st, hol_ed, Holiday1, Holiday2, Holiday3, Holiday4)".
                                              "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                            
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }                                              

            $this->log->set_log_device('schedule', 'insert', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("No"=>$No));
                                                                     
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch            
    }

    /** ==================================================================
     *  Schedule Update
     */
    public function update() {
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No             = $PutParams['No'];
            $Name           = strip_tags(trim(parent::InputDef($PutParams['Name'])));
            $Name           = str_replace("'", "", $Name);
            $Mean           = strip_tags(parent::InputDef($PutParams['Mean']));
            $Mean           = str_replace("'", "", $Mean);

            $SunReverseUse            = parent::InputDef($PutParams['SunReverseUse'], 0);        
            list($SunStartTime, $SunEndTime)  = self::parseInputTime(parent::InputDef($PutParams['SunTime'], '0000~0000'));
            $MonReverseUse            = parent::InputDef($PutParams['MonReverseUse'], 0);
            list($MonStartTime, $MonEndTime)  = self::parseInputTime(parent::InputDef($PutParams['MonTime'], '0000~0000'));
            $TueReverseUse            = parent::InputDef($PutParams['TueReverseUse'], 0);
            list($TueStartTime, $TueEndTime)  = self::parseInputTime(parent::InputDef($PutParams['TueTime'], '0000~0000'));
            $WedReverseUse            = parent::InputDef($PutParams['WedReverseUse'], 0);
            list($WedStartTime, $WebEndTime)  = self::parseInputTime(parent::InputDef($PutParams['WedTime'], '0000~0000'));
            $ThuReverseUse            = parent::InputDef($PutParams['ThuReverseUse'], 0);
            list($ThuStartTime, $ThuEndTime)  = self::parseInputTime(parent::InputDef($PutParams['ThuTime'], '0000~0000'));
            $FriReverseUse            = parent::InputDef($PutParams['FriReverseUse'], 0);
            list($FriStartTime, $FriEndTime)  = self::parseInputTime(parent::InputDef($PutParams['FriTime'], '0000~0000'));
            $SatReverseUse            = parent::InputDef($PutParams['SatReverseUse'], 0);
            list($SatStartTime, $SatEndTime)  = self::parseInputTime(parent::InputDef($PutParams['SatTime'], '0000~0000'));
            $HolidayReverseUse            = parent::InputDef($PutParams['HolidayReverseUse'], 0);
            list($HolidayStartTime, $HolidayEndTime)  = self::parseInputTime(parent::InputDef($PutParams['HolidayTime'], '0000~0000'));
            $Holiday1       = parent::InputDef($PutParams['Holiday1'], '0');
            $Holiday2       = parent::InputDef($PutParams['Holiday2'], '0');
            $Holiday3       = parent::InputDef($PutParams['Holiday3'], '0');
            $Holiday4       = parent::InputDef($PutParams['Holiday4'], '0');

            if(empty($No) || !is_numeric($No) || $No === '') {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }

            if(empty($Name) || 
            ($SunStartTime > $SunEndTime) || !is_numeric($SunStartTime) || !is_numeric($SunEndTime) ||
            ($MonStartTime > $MonEndTime)|| !is_numeric($MonStartTime) || !is_numeric($MonEndTime)||
            ($TueStartTime > $TueEndTime) || !is_numeric($TueStartTime) || !is_numeric($TueEndTime) ||
            ($WedStartTime > $WebEndTime) || !is_numeric($WedStartTime) || !is_numeric($WebEndTime) ||
            ($ThuStartTime > $ThuEndTime) || !is_numeric($ThuStartTime) || !is_numeric($ThuEndTime) ||
            ($FriStartTime > $FriEndTime) || !is_numeric($FriStartTime) || !is_numeric($FriEndTime) ||
            ($SatStartTime > $SatEndTime) || !is_numeric($SatStartTime) || !is_numeric($SatEndTime) ||
            ($HolidayStartTime > $HolidayEndTime) || !is_numeric($HolidayStartTime) || !is_numeric($HolidayEndTime)) { 
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }
            
            if((intval($SunStartTime) < 0 ) || (intval($SunStartTime) > 2359) ||
            (intval($SunEndTime) < 0 ) || (intval($SunEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($MonStartTime) < 0 ) || (intval($MonStartTime) > 2359) ||
            (intval($SunEndTime) < 0 ) || (intval($SunEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($TueStartTime) < 0 ) || (intval($TueStartTime) > 2359) ||
            (intval($TueEndTime) < 0 ) || (intval($TueEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($WedStartTime) < 0 ) || (intval($WedStartTime) > 2359) ||
            (intval($WebEndTime) < 0 ) || (intval($WebEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($ThuStartTime) < 0 ) || (intval($ThuStartTime) > 2359) || 
            (intval($ThuEndTime) < 0 ) || (intval($ThuEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($FriStartTime) < 0 ) || (intval($FriStartTime) > 2359) || 
            (intval($FriEndTime) < 0 ) || (intval($FriEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($SatStartTime) < 0 ) || (intval($SatStartTime) > 2359) || 
            (intval($SatEndTime) < 0 ) || (intval($SatEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

            if((intval($HolidayStartTime) < 0 ) || (intval($HolidayStartTime) > 2359) || 
            (intval($HolidayEndTime) < 0 ) || (intval($HolidayEndTime) > 2359)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;                 
            }

             /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
             $dbContext = $this->conn->prepare("SELECT count(No) FROM Schedule WHERE Site=? AND No=?");

             if(is_bool($dbContext)) {
                 throw new Exception($this->conn->errorInfo()[2]);
             }
 
             $dbContext->execute(array(parent::getAccountSite(), $No));
             $tmpRow = $dbContext->fetchAll();
             $dbContext = null; /* Database Lock Free */

             if($tmpRow == 0) {
                 parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                 exit;
             } 

             $valueArr = array($Name, $Mean, $SunReverseUse, $SunStartTime, $SunEndTime,
                               $MonReverseUse, $MonStartTime, $MonEndTime, $TueReverseUse, $TueStartTime, $TueEndTime, 
                               $WedReverseUse, $WedStartTime, $WebEndTime, $ThuReverseUse, $ThuStartTime, $ThuEndTime, 
                               $FriReverseUse, $FriStartTime, $FriEndTime, $SatReverseUse, $SatStartTime, $SatEndTime,
                               $HolidayReverseUse, $HolidayStartTime, $HolidayEndTime, 
                               $Holiday1, $Holiday2, $Holiday3, $Holiday4,parent::getAccountSite(), $No);

            $dbContext = $this->conn->prepare("UPDATE Schedule SET Name=?,mean=?, sun_reverse=?,sun_st=?,sun_ed=?, ".
                                              "mon_reverse=?, mon_st=?, mon_ed=?, tue_reverse=?, tue_st=?, tue_ed=?, ".
                                              "wed_reverse=?, wed_st=?, wed_ed=?, thu_reverse=?, thu_st=?, thu_ed=?,".
                                              "fri_reverse=?, fri_st=?, fri_ed=?, sat_reverse=?, sat_st=?, sat_ed=?,".
                                              "hol_reverse=?, hol_st=?, hol_ed=?, Holiday1=?, Holiday2=?, Holiday3=?, Holiday4=? ".
                                              " WHERE Site=? AND No=?");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                                
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */
            $this->log->set_log_device('schedule', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch            
    }

    /** ==================================================================
     *  Schedule 정보 삭제
     */    
    public function delete() {
        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        /* User No Check Valid */
        if(empty($No) || !is_numeric($No) || $No === '') {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM Schedule WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchAll();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 
            
            /* TODO 삭제 전 해당 Schedule를 사용 중인지 확인 후 사용중 이라면 Error 처리 */
            /* 관련 Table : Unlock Schedule (UnlockDoor), AccessLevel, EventAction, 
            */

            $dbContext = $this->conn->prepare("SELECT count(No) FROM AccessLevel WHERE Site=? And ScheduleNo=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow > 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0018);
                exit;
            } 

            $dbContext = $this->conn->prepare("DELETE FROM Schedule WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = null; /* Database Lock Free */
            
            $this->log->set_log_device('schedule', 'delete', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch

    }

    /** ==================================================================
    *  입력된 시간이 24:00 -> 23:5959 (System Time) 으로 변환 
    */      
    private function parseInputTime($time)
    {
        $parts = explode('~', $time);

        $start = $parts[0];
        $end   = $parts[1];

        $start = $start=='2400'?'2359':$start;  //$this->human_2400to2359($start);
        $end   = $end=='2400'?'2359':$end;  //$this->human_2400to2359($end);

        return array($start, $end);
    }    

    /** ==================================================================
     * License에 따른 최대 스케쥴 확인
     * TRUE : 최대 스케쥴 초과
     * FALSE : 최대 스케쥴 이하
     */
    private function checkMaxLimitedSchedule() {
        $maxSchedule = EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][12];
        if($this->util->GetRecordCount('Schedule') >= $maxSchedule) return TRUE;
        else FALSE;   
     }    
}

?>