<?php
/***********************************************************************************
 * FILE NAME : Log.php                                           DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2018.12.26 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 *        2    0.0.2       2019.11.25  CJMOON     add EventLogNum
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Log query
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class SysLog extends OpenApiBaseImpl
{   
    /* ================================================================== */    
    /* 지정된 Number 이후의 Log 조회 */
    public function EventLogNum()
    {
        $LastNo     = $this->input->get('No');
		$Timeout    = $this->input->get('Timeout', 0);
		$EventCodefilter = array_filter(explode(',',$this->input->get('Codefilter')));
        $LimitNum   = $this->input->get('Limit', 0);        
      
        if( empty($LastNo) || empty($LimitNum)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }

        if(!is_numeric($LastNo) ||!is_numeric($LimitNum) || !is_numeric($Timeout)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;
        }
		
		/* 최대 60초를 넘기지 못하도록 제한 함 */
		if(intval($Timeout) > 60) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;            
		}

        /* Limit는 최대 100개를 넘기지 못하도록 제한 함 */
        if(intval($LimitNum) > 100) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;            
        }
		
        try {			
            $inQuery = "";

			if(count($EventCodefilter) > 0) {
				
				$inQuery = "and log.eventCode IN (";
				for($i=0;$i<count($EventCodefilter);$i++) {
					$inQuery = $inQuery."'".$EventCodefilter[$i]."',";
				}

				$inQuery = rtrim($inQuery, ",");
				$inQuery = $inQuery.") ";
			}
            
            /*
            for($j=0;$j<2;$j++)
            {
				$query = " SELECT log.No, Type, eventCode, IFNULL(UserNo,'') userNo, IFNULL(UserName,'') AS userName, ". 
				" IFNULL(DeviceName,'') AS deviceName, strftime('%m-%d-%Y %H:%M:%S',datetime(date,'unixepoch','localtime')) AS eventDate, ".
				" strftime('%m-%d-%Y %H:%M:%S',datetime(ClientDate,'unixepoch','localtime')) AS localTime, IFNULL(Message,'') Message, IFNULL(Res1,'') Res1 ".
				" FROM log WHERE log.no>=? ".$inQuery." ORDER BY No ASC LIMIT ? ";			

				$list = $this->conn_log->prepare($query);
							
				if(is_bool($list)) {
					throw new Exception($this->conn_log->errorInfo()[2]);            
				}
				
				$list->execute(array($LastNo, $LimitNum));
				$list = $list->fetchAll(PDO::FETCH_ASSOC);

				// 조회된 건수가 없다면 Time Out 확인 후, 해당시간 만큼 지연 후, Retry
                if(count($list) == 0 && $j==0 && intval($Timeout)>0) 
                {
                    sleep($Timeout);
                } 
                else
                { 
                    break;				
                }
            }
            */

            for($j=0;$j<intval($Timeout);$j++)
            {
				$query = " SELECT log.No, Type, Port, eventCode, IFNULL(UserNo,'') userNo, IFNULL(UserName,'') AS userName, ". 
				" IFNULL(DeviceName,'') AS deviceName, strftime('%m-%d-%Y %H:%M:%S',datetime(date,'unixepoch','localtime')) AS eventDate, ".
				" strftime('%m-%d-%Y %H:%M:%S',datetime(ClientDate,'unixepoch','localtime')) AS localTime, IFNULL(Message,'') Message, IFNULL(Res1,'') Res1 ".
				" FROM log WHERE log.no>=? ".$inQuery." ORDER BY No ASC LIMIT ? ";			

				$list = $this->conn_log->prepare($query);
							
                if(is_bool($list))
                {
					throw new Exception($this->conn_log->errorInfo()[2]);            
				}
				
				$list->execute(array($LastNo, $LimitNum));
				$list = $list->fetchAll(PDO::FETCH_ASSOC);

				// 조회된 건수가 없다면 Time Out 확인 후, 해당시간 만큼 지연 후, Retry
                if(count($list) == 0) 
                {
                    sleep(1);
                } 
                else
                { 
                    break;				
                }
            }

					
            /* 국가별 언어에 따른 Log Message 처리 */
            if(empty($this->request_header['Lang'])) {            
                $str = file_get_contents(LANGUAGE_DIR.DS.'en/eventcode.json');
            } 
            else {
                $str = file_get_contents(LANGUAGE_DIR.DS.$this->request_header['Lang'].'/eventcode.json'); 
            }
            
            $json = json_decode($str, TRUE);
            
            foreach( $list as $key=>$val )
            {
                $val['Type'] = EnumTable::$attrLogType[$val['Type']];
                $val['EventDescription'] = @$json[$val['EventCode']];            
                $list[$key]	= $val;
            }

            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($list));            
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch     


    }

    /* ================================================================== */    
    /* 마지막 시간 기준으로 Log 조회 */
    public function LastEventLog()
    {        
        $no = 0;
        $spider_db_path = DATABASE_DIR.'/Spider.db';
        
        try {			        
            $list = $this->conn_log->prepare("ATTACH DATABASE '{$spider_db_path}' AS Spider");
        
			if(is_bool($list)) {
				throw new Exception($this->conn_log->errorInfo()[2]);
            }
            
            $list->execute();

            $query = " SELECT IFNULL(Site,0) As Site,No,IFNULL(Type,0) AS Type, IFNULL(Port,0) AS Port, IFNULL(EventCode,0) AS EventCode, " .
                     "  IFNULL(UserNo,0) AS UserNo,IFNULL(CardNo,0) AS CardNo,IFNULL(UserName,'') AS UserName,IFNULL(DeviceName,'') AS DeviceName, " .
                     "  strftime('%m-%d-%Y %H:%M:%S', datetime(Date,'unixepoch','localtime')) as dd, " .
                     "  strftime('%m-%d-%Y %H:%M:%S', datetime(ClientDate,'unixepoch','localtime')) as LocalTime, " .
                     "  IFNULL(Ack,0)  AS Ack, IFNULL(Message,'') AS Message, ClientMAC, Res1 " .
                     " FROM Log WHERE No > '".$no."' ORDER BY No ASC LIMIT 100";

            $list = $this->conn_log->prepare($query);

            if(is_bool($list)) {
                $this->conn_log-->exec("DETACH DATABASE '{$spider_db_path}' AS Spider");            
                throw new Exception($this->conn_log->errorInfo()[2]);            
            }

            $this->conn_log->exec("DETACH DATABASE '{$spider_db_path}' AS Spider");
            $output = array('time'=>date('m-d-Y H:i:s'), 'logs'=>'[]');

            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($output));

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                 
    }   
    
    /* ================================================================== */    
    /* 지정된 일자 시간의 데이타 조회 */
    public function EventLog()
    {
        $startDT = $this->input->get('startDate');
        $endDT	 = $this->input->get('endDate');        
      
        if( empty($startDT) || empty($endDT)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;                                        
        }

        /* 최대 검색기간 확인, Default MAX_SEARCH_DAY는 7일 (1주일) */
        $daygap = date_diff(new DateTime($startDT), new DateTime($endDT));
        if($daygap->days > MAX_SEARCH_DAY) {
            parent::toError(ConstOpenapiErrorCode::ERR_MAXSEARCH_DAYS);
            exit;             
        }

        try {
            $query = " SELECT log.No, Type, Port, eventCode, IFNULL(UserNo,'') userNo, IFNULL(UserName,'') AS userName, ". 
            " IFNULL(DeviceName,'') AS deviceName, strftime('%m-%d-%Y %H:%M:%S',datetime(date,'unixepoch','localtime')) AS eventDate, ".
            " strftime('%m-%d-%Y %H:%M:%S',datetime(ClientDate,'unixepoch','localtime')) AS localTime, IFNULL(Message,'') Message, IFNULL(Res1,'') Res1 ".
            " FROM log WHERE log.date>=? and log.date<=? ";

            $list = $this->conn_log->prepare($query);
                        
            if(is_bool($list)) {
                throw new Exception($this->conn_log->errorInfo()[2]);            
            }
            
            $list->execute(array(strtotime($startDT), strtotime($endDT)));
            $list   = $list->fetchAll(PDO::FETCH_ASSOC);           
                                   
            /* 국가별 언어에 따른 Log Message 처리 */
            if(empty($this->request_header['Lang'])) {            
                $str = file_get_contents(LANGUAGE_DIR.DS.'en/eventcode.json');
            } 
            else {
                $str = file_get_contents(LANGUAGE_DIR.DS.$this->request_header['Lang'].'/eventcode.json'); 
            }
            
            $json = json_decode($str, TRUE);
            
            foreach( $list as $key=>$val )
            {
                $val['Type'] = EnumTable::$attrLogType[$val['Type']];
                $val['EventDescription'] = @$json[$val['EventCode']];            
                $list[$key]	= $val;
            }

            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($list));            
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch           
    }
}
?>