<?php
/***********************************************************************************
 * FILE NAME : SystemInfo.php                                    DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2018.12.26 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2018.12.26  CJMOON     Init 
 * 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Log query
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class SystemInfo extends OpenApiBaseImpl
{
	/* ================================================================== */    
	/* 라이세스 정보 조회
	   결과 데이타 : 
		{
	"errCode": "0000",
	"errMsg": "Sucess",
	"data": [   {
		"Model": "3",
		"Type": "Door 4",
		"LicenseKey": "70B84C2E7ED69CA36E459A4486FC3875F483892782F04E9F53F57486BA5E1BC8",
		"Version": "5.00-00j",
		"ModelStr": "Enterprise",
		"MACAddress": "02:01:AE:2B:A4:ED"
	}]
	}
	*/
	function license()
	{
		try {			
			$list   = $this->net_work->prepare("SELECT Model, Type, LicenseKey FROM Network ORDER BY No DESC LIMIT ?, ?");
			if(is_bool($list)) {
				throw new Exception($this->net_work->errorInfo()[2]);
			}

			$list->execute(array(0, 10));            
			$list   = $list->fetchAll(PDO::FETCH_ASSOC);
		
			foreach( $list as $key=>$val ) 
			{
				$val['Version']  = $this->get_version();
				$val['Type']     = EnumTable::$attrModelType[$val['Model']][$val['Type']];
				$val['Model']    = EnumTable::$attrModel_NT[$val['Model']];
				$val['MACAddress']  = $this->get_macaddress();				
			}

			$list[$key] = $val;
			parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($list[$key]));
		} // try
		catch (PDOException $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		catch(Exception $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		
	}

	/* ================================================================== */    
	/* Server Network 조회, 보안을 위해 OpenAPI는 정보에서 제외
	   결과 데이타 : 
{
   "errCode": "0000",
   "errMsg": "Sucess",
   "data": [   {
      "IPType": "Static",
      "IPAddress": "192.168.20.124",
      "Subnet": "255.255.255.0",
      "Gateway": "192.168.20.1",
      "DNS1": "8.8.8.8",
      "DNS2": "",
      "HTTPPort": "80",
      "HTTPS": "On",
      "HTTPSPORT": "443"
   }]
}
	*/
	function network()
	{
		try {			
			$list   = $this->conn->prepare("SELECT IFNULL(IPType, 0) IPType, IPAddress, " .
										   "Subnet, Gateway, DNS1, DNS2, HTTPPort, IFNULL(HTTPS, '0') HTTPS, " .
										   " HTTPSPORT FROM NetworkInfo ORDER BY No DESC LIMIT ?, ?");
			if(is_bool($list)) {
				throw new Exception($this->conn->errorInfo()[2]);
			}

			$list->execute(array(0, 10));            
			$list   = $list->fetchAll(PDO::FETCH_ASSOC);
			
			foreach( $list as $key=>$val )
        	{
				if($val['IPType'] == '0') {
					$dhcp_info	= $this->dhcp_network();
					$val['IPAddress']	= $dhcp_info['addr'];
					$val['Subnet']	= $dhcp_info['mask'];
					$val['Gateway']	= $dhcp_info['bcast'];
				}

				$val['IPType'] = EnumTable::$attrIPType[$val['IPType']];
				$val['HTTPS']  = ($val['HTTPS'] == '0' ? 'Off' : 'on');
			}				
			
			$list[$key] = $val;
			parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($list[$key]));
		} // try
		catch (PDOException $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		catch(Exception $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		
	}

	/* ================================================================== */    
	/* DiskInfo
	*/	
	function systemused()
	{
	
		try {					
			$licensekey   = $this->net_work->query("SELECT LicenseKey, Model, Type FROM Network");
			if(is_bool($licensekey)) {
				throw new Exception($this->net_work->errorInfo()[2]);
			}

			$licensekey->execute();
			$licensekey = $licensekey->fetch(PDO::FETCH_ASSOC);

			$use	= 0;
			$use = $this->util->GetRecordCount("User");
			$rows[] = array(
				'name' => 'User',
				'total' => EnumTable::$attrModelSpec[$licensekey['Model']][$licensekey['Type']][2],
				'use' => (integer)$use
			);

			$use	= 0;
			$use = $this->util->GetRecordCount("Card");
			$rows[] = array(
				'name' => 'Card',
				'total' => EnumTable::$attrModelSpec[$licensekey['Model']][$licensekey['Type']][4],
				'use' => (integer)$use
			);

			$use	= 0;
			$use = $this->util->GetRecordCount("CardFormat");
			$rows[] = array(
				'name' => 'Card Format',
				'total' => EnumTable::$attrModelSpec[$licensekey['Model']][$licensekey['Type']][6],
				'use' => (integer)$use
			);

			$use	= 0;
			$use = $this->util->GetRecordCount("AccessLevel");
			$rows[] = array(
				'name' => 'Access Level',
				'total' => EnumTable::$attrModelSpec[$licensekey['Model']][$licensekey['Type']][11],
				'use' => (integer)$use
			);

			$use	= 0;
			$use = $this->util->GetRecordCount("Schedule");
			$rows[] = array(
				'name' => 'Schedule',
				'total' => EnumTable::$attrModelSpec[$licensekey['Model']][$licensekey['Type']][12],
				'use' => (integer)$use
			);
			
			$year   = date('Y');
			$search_year_start = self::to_timestamp("01-01-{$year}", 0, 0, 0);
			$search_year_end = self::to_timestamp("12-31-{$year}", 23, 59, 59);
			$holiday_fields = array($search_year_start, $search_year_end);

			$use	= 0;
			$use = $this->util->GetRecordCountSet("Holiday", "WHERE StartTime BETWEEN ? AND ?", $holiday_fields);
			$rows[] = array(
				'name' => 'Holiday',
				'total' => $this->GetLimitCount( ConstTable::MAX_HOLIDAY_1, ConstTable::MAX_HOLIDAY_2, ConstTable::MAX_HOLIDAY_3,ConstTable::MAX_HOLIDAY_4 ),
				'use' => (integer)$use
			);
			
			$use	= 0;
			$user_def	= $this->conn->prepare("SELECT * FROM UserDefined WHERE Site=?");
			$user_def->execute(array(parent::getAccountSite()));

			if( $user_def = $user_def->fetch(PDO::FETCH_ASSOC) ) {
				foreach( $user_def as $name=>$val ) {
					if( substr($name, 0, 4) == 'User' && !empty($val) ) {
						$use++;
					}
				}
			}

			$rows[] = array(
				'name' => 'User Def.Field',
				'total' => $this->GetLimitCount( ConstTable::MAX_USERDEFINE_1, ConstTable::MAX_USERDEFINE_2, ConstTable::MAX_USERDEFINE_3,ConstTable::MAX_USERDEFINE_4 ),
				'use' => $use
			);

			$vars = array(
				'rows' => $rows,
				'Transaction'	=> $this->util->get_logstatus(),
				'Disk Space'	=> $this->util->get_diskspace(),
				'User Image'	=> $this->util->get_userspace(),			
				'SD Storage'    => $this->util->get_sdspace(),
				'USB Storage'   => $this->util->get_usbspace()
			);				

			parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($vars));
		} // try
		catch (PDOException $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch
		catch(Exception $e) {
			parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
			exit;
		} // catch		
	}


	/* ================================================================== */    
	/* DHCP인 경우 설정된 내부 네트워크 정보 조회
	*/	
	private function dhcp_network()
	{
		exec(SPIDER_COMM.' ifconfig', $lines);
		$output	= implode("\n", $lines);

		$match  = "/^.*addr:([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})";
		$match .= ".*Bcast:([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})";
		$match .= ".*Mask:([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}).*$/im";
		if(!preg_match($match, $output, $matches))
			return false;

		$values = array('addr'	=> $matches[1], 'mask'	=> $matches[3] );

		$lines = '';
		$output = array();
		$matches = array();

		exec(SPIDER_COMM.' getroute', $lines);
		
		foreach( $lines as $line )
		{
			$line		= strtolower(trim($line));
			$action		= explode(':', $line);
			$status		= trim(@$action[0]);

			if( strpos($status, 'result') === 0 ) {
				$values['bcast']	= trim($action[1]);
				break;
			}
		}

		return $values;
	}

/*
	function get() 
	{
        $url = urldecode($_SERVER['REQUEST_URI']);

        $list   = $this->net_work->prepare("SELECT * FROM Network ORDER BY No DESC LIMIT ?, ?");
        $list->execute(array(0, 10));
        $list   = $list->fetchAll(PDO::FETCH_ASSOC);

        foreach( $list as $key=>$val )
        {
        	$val['VersionStr']  = $this->get_version();
            $val['ModelStr']    = EnumTable::$attrModel[$val['Model']];
			$val['TypeStr']     = EnumTable::$attrModelType[$val['Model']][$val['Type']];
			$val['MACAddress']  = $this->get_macaddress();
			
			$list[$key] = $val;
        }    

        echo json_encode($list);
    }
*/
	/* ================================================================== */    
	/* Spider-cgi의 Version정보 조회 */
	private function get_version() 
	{ 
		exec(SPIDER_COMM." ver", $output);
		$version	= @$output[count($output)-2];        
		return $version;
    }

	/* ================================================================== */    
	/* 현재의 MAC Address */	
    private function get_macaddress()
	{
		$mac = "";
		exec(SPIDER_COMM." getmac", $output);

		foreach( $output as $line ) {
			$line = trim($line);
			$name_split = explode('-', $line);

			if( $name_split[0] == 'MAC' ) 
			{
				$mac = $name_split[1];
			}
		}
		
		return $mac;
	}

	/* ================================================================== */    
	/* 현재시간 반환 */	
	private function to_timestamp($date="", $hour=0, $min=0, $sec=0)
    {
        if ($date != "") {
        	$timeval = explode('-', trim($date));
        	
        	if (count($timeval) < 3) {
        		return "";
        	}
        		
            $month  = $timeval[0];
            $day    = $timeval[1];
            $year   = $timeval[2];
            return mktime($hour, $min, $sec, $month, $day, $year);
        }
        return $date;
	}
	
	/* ================================================================== */
	/* 라이센스에 따른 최대 사용가능 계산 */
	private function GetLimitCount($num1, $num2, $num3, $num4)
	{
		if ($this->neptune_model == ConstTable::MODEL_ESSENTIAL)
    		return $num1;
    	if ($this->neptune_model == ConstTable::MODEL_ELITE)
    		return $num2;
    	if ($this->neptune_model == ConstTable::MODEL_ENTERPRISE)
			return $num3;
		if ($this->neptune_model == ConstTable::MODEL_GIANT)
    		return $num4;			
        
        return 0;
	}
	
}
?>
