<?php
/***********************************************************************************
 * FILE NAME : Threat.php                                        DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.05.22 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.05.22  CJMOON     Init 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Threat Level Setting API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";

class Threat extends OpenApiBaseImpl
{
    /** ==================================================================
     *  현재의 Threat 상태 정보 반환
     */
    public function clist() {
    try {
        $dbContext = $this->conn->prepare("SELECT ThreatStatus, ThreatCount FROM Threat WHERE Site=?");

        if(is_bool($dbContext)) {
            throw new Exception($this->conn->errorInfo()[2]);
        }

        $dbContext->execute(array(parent::getAccountSite()));
        $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

        if(count($dbContext) > 0) {
            if(strlen($dbContext[0]['ThreatStatus']) == 0) $dbContext[0]['ThreatStatus'] = '0'; /* OFF STATUS */
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, $dbContext);
        } 
        else {
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }   

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /** ==================================================================
     *  Threat Level Setting
     *  주의 사항 : Paramter Value가 0으로 사용 할 경우 empty() 사용 금지
     *             empty('0')으로 사용 할 경우 TRUE로 반환 함.
     */
    public function update() {
        $ThreatCount = "0";
        $No = "";

        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

    try {
        $ThreatLevel = $PutParams['Level'];
                
        if($ThreatLevel === '' || !is_numeric($ThreatLevel)) {        
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;            
        }
        
        $list = $this->conn->prepare("Select ThreatStatus From Threat WHERE Site=? AND No=1");
        $list->execute(array($_SESSION['spider_site']));
        $list = $list->fetchAll(PDO::FETCH_ASSOC);
        
        foreach( $list as $key=>$val)
        {
            $check_value = $val['ThreatStatus'];
        }

        /* Key Create */
        $dbContext = $this->conn->prepare("SELECT (IFNULL(MAX(No), 0)+1) No,  ThreatCount FROM Threat WHERE Site=?");
        if(is_bool($dbContext)) {
           throw new Exception($this->conn->errorInfo()[2]);
        }

        $dbContext->execute(array(parent::getAccountSite()));
        $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);
       
        $No = $dbContext[0]['No'];
        if($No =='1') { /* 기존 데이타가 없으므로 INSERT 처리 */                                
            $dbContext = $this->conn->prepare("INSERT INTO Threat (Site,No,ThreatStatus,ThreatCount) VALUES ( ?,1,0,5)");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }                
            $dbContext->execute(array(parent::getAccountSite()));
            $ThreatCount = "5";
        } 
        else $ThreatCount = $dbContext[0]['ThreatCount'];

        if((int)$ThreatLevel > (int)$ThreatCount) {
            parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_THREAT_LEVEL);
            exit;  
        }

        $dbContext = $this->conn->prepare("UPDATE Threat SET ThreatStatus=? WHERE Site=? AND No=1");
        if(is_bool($dbContext)) {
           throw new Exception($this->conn->errorInfo()[2]);
        }

        if(!$dbContext->execute(array($ThreatLevel, parent::getAccountSite()))) {
            throw new Exception($this->conn->errorInfo()[2]);
        }
        $dbContext = null; /* Database Lock Free */
        
        
        if($ThreatLevel < $check_value)
        {        
            exec(SPIDER_COMM." threatlv locksetup");
        }
        else
        {
            exec(SPIDER_COMM." threatlv keepsetup");
        }
        

        $this->log->set_log_device('atlevel', 'update', '0', '', OPENAPI_ID);
        parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
    }

    
}

?>