<?php
/***********************************************************************************
 * FILE NAME : UnlockSchedule.php                                DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.05.28 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.05.28  CJMOON     Init 
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Unlock Schedule Setting API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';

class UnlockSchedule extends OpenApiBaseImpl
{
    /** ==================================================================
     *  현재의 Unlock List 정보 반환
     *  Reverse Field를 ScheduleNo용으로 사용 중임.
     */
    public function clist() {
    try {
        $dbContext = $this->conn->prepare("SELECT No, Name, Reverse ScheduleNo, SelectType FROM UnlockSchedule WHERE Site=?");

        if(is_bool($dbContext)) {
            throw new Exception($this->conn->errorInfo()[2]);
        }

        $dbContext->execute(array(parent::getAccountSite()));
        $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

        foreach ($dbContext as $key=>$val) {
            $val['DoorList'] = self::to_list_UnlockDoor($val['No']);
            $dbContext[$key] = $val;
        }

        if(count($dbContext) > 0) {
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, $dbContext);
        } 
        else {
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        }  

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch            
        
    }

    /* ================================================================== */
    /* 신규 Unlock Schedule */
    public function insert() {
        $Name       = strip_tags(trim($this->input->post('Name')));
        $ScheduleNo = $this->input->post('ScheduleNo', 0);
        $SelectType = $this->input->post('SelectType');    
        $DoorList   = array_filter(explode(',',$this->input->post('DoorList')));

        try {            
            if(empty($Name) || $Name === '' || 
            $ScheduleNo ==='' || !is_numeric($ScheduleNo) || empty($ScheduleNo) || 
            $SelectType ==='' || !is_numeric($SelectType) || empty($SelectType) || 
            count($DoorList) == 0) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
            exit;     
            }        

            /* User Key Create */
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM UnlockSchedule WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                
            if(!$dbContext->execute(array(parent::getAccountSite()))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                    
            $No = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            /* SelectType - 1 : Door Individual (개별), 2 : Door Group, 3: Elevator, 4: AUX Output 
             * Reverse Field를 ScheduleNo용으로 사용 중임.
             */
            $valueArr = array(parent::getAccountSite(), $No, $Name, $ScheduleNo, $SelectType);
            $dbContext = $this->conn->prepare("INSERT INTO UnlockSchedule (Site, No, Name, Reverse, SelectType) VALUES (?,?,?,?,?)");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                        
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            } 

            error_log('DB Error');
            self::insertUnlockDoor($No, $DoorList);

            
            $this->log->set_log_device('unlockschedule', 'insert', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("No"=>$No));
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch              
    }

    /** ==================================================================
     *  Unlock Schedule Update
     */    
    public function update() {
         /* PHP PUT 처리 */
         parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No         = $PutParams['No'];
            $Name       = strip_tags(trim($PutParams['Name']));
            $ScheduleNo = $PutParams['ScheduleNo'];
            $SelectType = $PutParams['SelectType'];    
            $DoorList   = array_filter(explode(',',$PutParams['DoorList']));

            if(empty($No) || !is_numeric($No) || $No === '') {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }

            if(empty($Name) || $Name === '' || 
                $ScheduleNo ==='' || !is_numeric($ScheduleNo) || empty($ScheduleNo) || 
                $SelectType ==='' || !is_numeric($SelectType) || empty($SelectType) || 
                count($DoorList) == 0) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_VALUE);
                exit;     
            }         
         
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM UnlockSchedule WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchAll();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            }

            /* SelectType - 1 : Door Individual (개별), 2 : Door Group, 3: Elevator, 4: AUX Output 
             * Reverse Field를 ScheduleNo용으로 사용 중임.
             */
            $valueArr = array($Name, $ScheduleNo, $SelectType, parent::getAccountSite(), $No);
            $dbContext = $this->conn->prepare("UPDATE UnlockSchedule SET Name=?,Reverse=?, SelectType=? WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                                            
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

             /* 기존 UnlockDoo 삭제 */
            self::deleteUnlockDoor($No);
             /* 새로운 UnlockDoo로 재입력 */
            self::insertUnlockDoor($No, $DoorList);

            $this->log->set_log_device('unlockschedule', 'update', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch 
    }

    /** ==================================================================
     *  unlockschedule Delete
     */     
    public function delete() {
        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        /* User No Check Valid */
        if(empty($No) || !is_numeric($No) || $No === '') {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* 기존 데이타를 검색하여 데이타의 일치성 확인 */
            $dbContext = $this->conn->prepare("SELECT count(No) FROM UnlockSchedule WHERE Site=? AND No=?");

            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $tmpRow = $dbContext->fetchColumn();
            $dbContext = null; /* Database Lock Free */

            if($tmpRow == 0) {
                parent::toJson(ConstOpenapiErrorCode::ERR_INPUT_VALUE_MISMATCH);
                exit;
            } 

            /* 기존 Access Level 삭제 */
            self::deleteUnlockDoor($No);

            $dbContext = $this->conn->prepare("DELETE FROM UnlockSchedule WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = null; /* Database Lock Free */
            
            $this->log->set_log_device('unlockschedule', 'delete', '0', '', OPENAPI_ID);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }
    /* ================================================================== */   
    /* Unlock 대상 List 추출 */
    private function  to_list_UnlockDoor($UnlockNo) {
        $list   = $this->conn->prepare("SELECT ElementNo FROM UnlockDoor WHERE Site=? AND UnlockNo=?");
        $list->execute(array(parent::getAccountSite(), $UnlockNo));
        $list = $list->fetchAll(PDO::FETCH_COLUMN);
        $UnlockList = implode(", ", $list);
        
        return $UnlockList;
    }

/** ==================================================================
     * Access Level Reader Insert
     * INSERT, UPDATE 공용 으로 사용.
     */
    private function insertUnlockDoor($UnLockNo, $DoorList) {
        try {
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM UnlockDoor WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                            
            $dbContext->execute(array(parent::getAccountSite()));
            $No = $dbContext->fetchColumn(); 

            $dbContext = $this->conn->prepare("INSERT INTO UnlockDoor (Site, No, UnlockNo, ElementNo) VALUES (?,?,?,?)");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            foreach( $DoorList as $value ) {                
                $valueArr = array(parent::getAccountSite(), $No, $UnLockNo, $value);
                if(!$dbContext->execute($valueArr)) {
                    throw new Exception($this->conn->errorInfo()[2]);
                }

                $No = $No+1;
            }
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
    }    

    /** ==================================================================
     *  UnlockDoor를 삭제 시 사용 (Update 시에도 사용 - 기존 데이타 삭제)
     */
    private function deleteUnlockDoor($UnLockNo) {
        try {
            $dbContext = $this->conn->prepare("DELETE FROM UnlockDoor WHERE Site=? AND UnlockNo=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $UnLockNo));
        }
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                
    }    

}
?>    