<?php
/***********************************************************************************
 * FILE NAME : User.php                                        DESIGN REF : FMCM00
 * 
 * AUTHOR  : CHEOLJOO MOON
 * DATE    : 2019.01.03 (KR, UTC+9)
 * CHANGES :
 *     REF NO  VERSION     DATE        WHO        DETAIL 
 *        1    0.0.1       2019.01.03  CJMOON     Init 
 *        2    0.0.1       2019.05.15  CJMOON     ADD Card Holder List, Insert
 * FILE Encoding : UTF-8
 * 
 * DESCRIPTION : Card Holder API
 *  
 ***********************************************************************************/
require_once "./common/OpenApiBaseImpl.php";
require_once "./common/ConstOpenapiErrorCode.php";
require_once APP_DIR.'/libraries/EnumTable.php';
require_once APP_DIR.'/libraries/ConstTable.php';

class User extends OpenApiBaseImpl
{
    /* ================================================================== */    
    /* User Lsit 정보 */
    public function clist()
    {
        $page   = $this->input->get('page', 1);   /* 요청 할 Page 번호 */

        try {
            $dbContext = $this->conn->prepare("SELECT COUNT(No) FROM User WHERE Site=?");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
             }

            $dbContext->execute(array(parent::getAccountSite()));
            $totalUser = (int)$dbContext->fetchColumn();        

            $dbContext = $this->conn->prepare("SELECT No, FirstName, MiddleName, LastName, Phone, CellPhone, Email, Ada, ThreatLevel, Exempt, WEBLevel, ImageFile, " .
                                              " UserDefine1, UserDefine2, UserDefine3, UserDefine4, UserDefine5, UserDefine6, UserDefine7, UserDefine8, UserDefine9, UserDefine10, " .
                                              " UserDefine11, UserDefine12, UserDefine13, UserDefine14, UserDefine15, UserDefine16, UserDefine17, UserDefine18, UserDefine19, UserDefine20 " .
                                              " from User WHERE Site=? ORDER BY NO DESC LIMIT ?,?");

            if(is_bool($dbContext)) {
              throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), parent::getCurrentPaging($page), MAX_LIST_ROW));  /* Reocrd를 20개 단위로 사용 */
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);

            
            /*
            foreach( $dbContext as $key=>$val )
            {
                $lv_val = $val['WEBLevel'];
                $name = $this->conn->prepare("SELECT Name FROM WebUser WHERE Site=? AND No=?");
                $name->execute(array(parent::getAccountSite(),$lv_val));
                $name = $name->fetchColumn();
                if(!$name)
                {
                    $name="";
                }
                $val['WebUserAccount']    = $name;
                $dbContext[$key] = $val;
            }
            */
            
            if(count($dbContext) > 0) {
                $vars = array( "total"=>$totalUser, "totalPage"=>parent::getPaging($totalUser), 'rows' => $dbContext);
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array($vars));
            } 
            else {
                parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
            }   

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /* ================================================================== */  
    /** User Information Insert
     * 
     */
    public function insert()
    {        
        $imageFileName = "";

        try {
            $firstName   = strip_tags(trim($this->input->post('FirstName')));
            $middleName  = strip_tags(trim($this->input->post('MiddleName')));
            $lastName    = strip_tags(trim($this->input->post('LastName')));
            $phone       = strip_tags(trim($this->input->post('Phone')));
            $cellphone   = strip_tags(trim($this->input->post('CellPhone')));
            $email       = strip_tags(trim($this->input->post('Email')));            
            $ada         = $this->input->post('Ada','0');
            $threatLevel = $this->input->post('ThreatLevel','0');
            $exempt      = $this->input->post('Exempt','0');
            
            $userDefine1 = $this->input->post('UserDefine1');
            $userDefine2 = $this->input->post('UserDefine2');
            $userDefine3 = $this->input->post('UserDefine3');
            $userDefine4 = $this->input->post('UserDefine4');
            $userDefine5 = $this->input->post('UserDefine5');
            $userDefine6 = $this->input->post('UserDefine6');
            $userDefine7 = $this->input->post('UserDefine7');
            $userDefine8 = $this->input->post('UserDefine8');
            $userDefine9 = $this->input->post('UserDefine9');
            $userDefine10 = $this->input->post('UserDefine10');
            $userDefine11 = $this->input->post('UserDefine11');
            $userDefine12 = $this->input->post('UserDefine12');
            $userDefine13 = $this->input->post('UserDefine13');
            $userDefine14 = $this->input->post('UserDefine14');
            $userDefine15 = $this->input->post('UserDefine15');
            $userDefine16 = $this->input->post('UserDefine16');
            $userDefine17 = $this->input->post('UserDefine17');
            $userDefine18 = $this->input->post('UserDefine18');
            $userDefine19 = $this->input->post('UserDefine19');
            $userDefine20 = $this->input->post('UserDefine20');
            
            /* 등록일자 */
            $enrollmentDate = date('YmdHis');

            /* Check Valid */
            if(empty($firstName) || 
               empty($lastName)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0011);
                exit;            
            }

            /* Check Ada */
            if($ada != '1') $ada ="0";

            /* Check ThreatLevel */
            if($threatLevel === '0') $threatLevel = "1";

            if($this->checkMaxLimitedUser()) {            
                parent::toError(ConstOpenapiErrorCode::ERR_LIMITED_CARD_HOLDER);
                exit;            
            }
            
            /* User Key Create */
            $dbContext = $this->conn->prepare("SELECT IFNULL(MAX(No), 0)+1 FROM User WHERE Site=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
                            
            $dbContext->execute(array(parent::getAccountSite()));
            $No = $dbContext->fetchColumn();  
            $dbContext = null; /* Database Lock Free */

            /* Image File Upload Use Check */
            if(count($_FILES) > 0) {
                $imageFileName = $this->ImageUpLoad($No);
                if(is_bool($imageFileName)) {                    
                    parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0012);
                    exit;                                
                }            
            }

            $valueArr = array(parent::getAccountSite(), $No, $firstName, $middleName, $lastName, $phone, $cellphone, $email, $ada, $threatLevel, 
                              $exempt, $enrollmentDate, $imageFileName,
                              $userDefine1, $userDefine2, $userDefine3, $userDefine4, $userDefine5, $userDefine6, $userDefine7,
                              $userDefine8, $userDefine9, $userDefine10,$userDefine11,$userDefine12,$userDefine13,$userDefine14,
                              $userDefine15,$userDefine16,$userDefine17,$userDefine18,$userDefine19,$userDefine20);

            $dbContext = $this->conn->prepare("INSERT INTO User (Site, No, FirstName, Middlename, LastName, Phone, CellPhone, Email, Ada,ThreatLevel, ". 
                                             "Exempt, Date, ImageFile, ".
                                             "UserDefine1,UserDefine2,UserDefine3,UserDefine4,UserDefine5,UserDefine6,UserDefine7,".
                                             "UserDefine8,UserDefine9,UserDefine10,UserDefine11,UserDefine12,UserDefine13,UserDefine14,".
                                             "UserDefine15,UserDefine16,UserDefine17,UserDefine18,UserDefine19,UserDefine20) ".
                                             " VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

            $this->log->set_log_device('card_holder', 'insert', '0', '', OPENAPI_ID,$FirstName .' '. $LastName);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS, array("Date"=>$enrollmentDate, 
                                                                     "No"=>$No, 
                                                                     "Ada"=>$ada, 
                                                                     "ThreatLevel"=>$threatLevel,
                                                                     "Exempt"=>$exempt));
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /** ==================================================================
     *  User Information Update
     */
    public function update()
    {
        $imageFileName = "";
        $oldImageFileName = "";
        
        /* PHP PUT 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        try {
            $No          = $PutParams['No'];
            $firstName   = strip_tags(trim($PutParams['FirstName']));
            $middleName  = strip_tags(trim($PutParams['MiddleName']));
            $lastName    = strip_tags(trim($PutParams['LastName']));
            $phone       = strip_tags(trim($PutParams['Phone']));
            $cellphone   = strip_tags(trim($PutParams['CellPhone']));
            $email       = strip_tags(trim($PutParams['Email']));
            $ada         = $PutParams['Ada'];
            $threatLevel = $PutParams['ThreatLevel'];
            $exempt      = $PutParams['Exempt'];
            
            $userDefine1 = strip_tags(trim($PutParams['UserDefine1']));
            $userDefine2 = strip_tags(trim($PutParams['UserDefine2']));
            $userDefine3 = strip_tags(trim($PutParams['UserDefine3']));
            $userDefine4 = strip_tags(trim($PutParams['UserDefine4']));
            $userDefine5 = strip_tags(trim($PutParams['UserDefine5']));
            $userDefine6 = strip_tags(trim($PutParams['UserDefine6']));
            $userDefine7 = strip_tags(trim($PutParams['UserDefine7']));
            $userDefine8 = strip_tags(trim($PutParams['UserDefine8']));
            $userDefine9 = strip_tags(trim($PutParams['UserDefine9']));
            $userDefine10 = strip_tags(trim($PutParams['UserDefine10']));
            $userDefine11 = strip_tags(trim($PutParams['UserDefine11']));
            $userDefine12 = strip_tags(trim($PutParams['UserDefine12']));
            $userDefine13 = strip_tags(trim($PutParams['UserDefine13']));
            $userDefine14 = strip_tags(trim($PutParams['UserDefine14']));
            $userDefine15 = strip_tags(trim($PutParams['UserDefine15']));
            $userDefine16 = strip_tags(trim($PutParams['UserDefine16']));
            $userDefine17 = strip_tags(trim($PutParams['UserDefine17']));
            $userDefine18 = strip_tags(trim($PutParams['UserDefine18']));
            $userDefine19 = strip_tags(trim($PutParams['UserDefine19']));
            $userDefine20 = strip_tags(trim($PutParams['UserDefine20']));
            

            /* User No Check Valid */
            if(empty($No) || !is_numeric($No)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
                exit;            
            }

            /* Check Valid */
            if(empty($firstName) || empty($lastName)) {
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0011);
                exit;            
            }

            /* Check Ada */
            if(!is_numeric($ada) || strlen($ada)>1 ||
                strlen($ada)==0  || empty($ada)) $ada ="0";

            /* Check Ada */
            if($ada != '1') $ada ="0";
            
            /* Check ThreatLevel */
            if(!is_numeric($threatLevel) ||  strlen($threatLevel)>1 ||
                strlen($threatLevel)==0  || empty($threatLevel)) $threatLevel ="1";

            if(!is_numeric($exempt) || strlen($exempt)>1 ||
                strlen($exempt)==0  || empty($exempt)) $exempt ="0";
                                                               
            /* Upload되는 파일이 존재 한다면 새로운 파일명으로 사용, 없다면 기존 파일명으로 Update처리 */
            $valueArr = array($firstName, $middleName, $lastName, $phone, $cellphone, $email, $ada, $threatLevel, $exempt, 
                              $userDefine1, $userDefine2, $userDefine3, $userDefine4, $userDefine5, $userDefine6, $userDefine7,
                              $userDefine8, $userDefine9, $userDefine10,$userDefine11,$userDefine12,$userDefine13,$userDefine14,
                              $userDefine15,$userDefine16,$userDefine17,$userDefine18,$userDefine19,$userDefine20,
                              parent::getAccountSite(), $No);

            $dbContext = $this->conn->prepare("UPDATE User SET FirstName=?, Middlename=?, LastName=?, Phone=?, CellPhone=?, Email=?, Ada=?,ThreatLevel=?, Exempt=?, ".
                                             "UserDefine1=?,UserDefine2=?,UserDefine3=?,UserDefine4=?,UserDefine5=?,UserDefine6=?,UserDefine7=?,".
                                             "UserDefine8=?,UserDefine9=?,UserDefine10=?,UserDefine11=?,UserDefine12=?,UserDefine13=?,UserDefine14=?,".
                                             "UserDefine15=?,UserDefine16=?,UserDefine17=?,UserDefine18=?,UserDefine19=?,UserDefine20=? ".
                                             " WHERE Site=? AND No=? ");
            
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            if(!$dbContext->execute($valueArr)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $this->log->set_log_device('card_holder', 'update', '0', '', OPENAPI_ID,$FirstName .' '. $LastName);

            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
    }

    /** ==================================================================
     *  User Information Delete (사용자 삭제 시 관련 데이타 동반 삭제 처리)
     *  Card Holder 삭제 순서 1. CardAccessLevel, 2. Card, 3 GroupElement, 
     *                       4. User, 5 Image File
     */
    public function delete() {
        $ImageFileName = "";

        /* PHP DELETE 처리 */
        parse_str(file_get_contents("php://input"), $PutParams);

        $No = $PutParams['No'];
        /* User No Check Valid */
        if(empty($No) || !is_numeric($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {

            /* User Image File Name Check */
            $dbContext = $this->conn->prepare("SELECT IFNULL(ImageFile,'') FROM User WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $imageFileName = $dbContext->fetchColumn();  
            $dbContext = null; /* Database Lock Free */

            /* Card Access Level Delete */
            $dbContext = $this->conn->prepare("DELETE FROM cardaccesslevel WHERE cardno IN (SELECT no FROM Card WHERE site=? AND userno=?)");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));

            /* GroupElement */
            $dbContext = $this->conn->prepare("DELETE FROM GroupElement WHERE Site=? AND GroupKind=? AND ElementNo=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), ConstTable::GROUP_USER, $No));

            /* Card Delete */
            $dbContext = $this->conn->prepare("DELETE FROM Card WHERE Site=? AND UserNo=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));

            /* User */
            $dbContext = $this->conn->prepare("DELETE FROM User WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));

            /* Image File Delete */
            if(strlen($imageFileName) > 0) $this->deleteImageFile($imageFileName);

            $this->log->set_log_device('card_holder', 'delete', '0', '', OPENAPI_ID,$FirstName .' '. $LastName);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);
        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch

    }

    /** ==================================================================
     *  Image File Update 명령, Image File의 변경만 가능
     */
    public function changeImage() {
        $No         = $this->input->post('No');

        /* User No Check Valid */
        if(empty($No) || !is_numeric($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        /* Image File Upload Use Check */
        if(count($_FILES) > 0) {
            $imageFileName = $this->ImageUpLoad($No);
            if(is_bool($imageFileName)) {                    
                parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0012);
                exit;                                
            }            
        }
        else { /* Upload된 파일이 존재 않는 다면 삭제 모드 */
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0012);
            exit;            
        }

        try {
            /* User Image File Name Check */
            $dbContext = $this->conn->prepare("SELECT IFNULL(ImageFile,'') ImageFile, FirstName, LastName FROM User WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);  
            $oldImageFileName = $dbContext[0]['ImageFile'];
            $FirstName = $dbContext[0]['FirstName'];
            $LastName = $dbContext[0]['LastName'];

            $dbContext = $this->conn->prepare("UPDATE User SET ImageFile=? WHERE Site=? AND No=? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            if(!$dbContext->execute(array($imageFileName,parent::getAccountSite(), $No))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

            /* Image File Delete */
            if(strlen($oldImageFileName) > 0) $this->deleteImageFile($oldImageFileName);

            $this->log->set_log_device('card_holder', 'update', '0', '', OPENAPI_ID,$FirstName .' '. $LastName);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch        
                
    }

    /** ==================================================================
     *  Image File Update 명령, Image File의 삭제만 가능
     */
    public function deleteImage() {
        $No         = $this->input->post('No');

        /* User No Check Valid */
        if(empty($No) || !is_numeric($No)) {
            parent::toError(ConstOpenapiErrorCode::ERR_INPUT_REQUIRED_0013);
            exit;            
        }

        try {
            /* User Image File Name Check */
            $dbContext = $this->conn->prepare("SELECT IFNULL(ImageFile,'') ImageFile, FirstName, LastName FROM User WHERE Site=? AND No=?");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }

            $dbContext->execute(array(parent::getAccountSite(), $No));
            $dbContext = $dbContext->fetchAll(PDO::FETCH_ASSOC);  
            $oldImageFileName = $dbContext[0]['ImageFile'];
            $FirstName = $dbContext[0]['FirstName'];
            $LastName = $dbContext[0]['LastName'];

            $dbContext = $this->conn->prepare("UPDATE User SET ImageFile='' WHERE Site=? AND No=? ");
            if(is_bool($dbContext)) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            if(!$dbContext->execute(array(parent::getAccountSite(), $No))) {
                throw new Exception($this->conn->errorInfo()[2]);
            }
            
            $dbContext = null; /* Database Lock Free */

            /* Image File Delete */
            if(strlen($oldImageFileName) > 0) $this->deleteImageFile($oldImageFileName);

            $this->log->set_log_device('card_holder', 'update', '0', '', OPENAPI_ID,$FirstName .' '. $LastName);
            parent::toJson(ConstOpenapiErrorCode::ERR_SUCCESS);

        } // try
        catch (PDOException $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch
        catch(Exception $e) {
            parent::toError(ConstOpenapiErrorCode::ERR_ETC, $e->getMessage());
            exit;
        } // catch                                
    }

    /** ==================================================================
     *  조건 : 입력된 파일명 존재 여부를 확인 후 해당 파일이 존재 한다면 삭제
     */      
    private function deleteImageFile($fileName) {
        if(file_exists(API_USER_IMG.'/'.$fileName)) 
            unlink(API_USER_IMG.'/'.$fileName);
    }
    /** ==================================================================
     *  조건 : 기존 파일명과 동일 하다면, 다름 이름으로 사용 해야 한다.
     *        string : 정상적으로 처리 되고, 변환된 파일명
     *        FALSE  :  Upload 실패
     *  Image File UpLoad
     */
    private function ImageUpLoad($No, $maxImageSize=MAX_IMAGE_SIZE) {
       
       /* File Name 또는 Image File Size Check */
       if(empty($_FILES['uploadFile']['name'])          ||
         (intval($_FILES['uploadFile']['size']) === 0 ) || 
         (intval($_FILES['uploadFile']['size']) > $maxImageSize)) {
          return FALSE;
       }
       
       /* Upload File Check */
       if( !is_uploaded_file($_FILES['uploadFile']['tmp_name'])) {
            error_log('is_uploaded_file failed');
            return FALSE;
       }

       /* 파일명 중간에 Blank 존재 시 치환 */
       $tagetFile     = str_replace(' ', '_', $_FILES['uploadFile']['name']);
       /* 파일 확장자 검사 */
       $tagetFileExt  = pathinfo($tagetFile, PATHINFO_EXTENSION);
       $allowedExt    = 'jpg|jpeg|gif|png|bmp';
       
       $checkFileExt  = explode('|', strtolower($allowedExt));
       if( !in_array(strtolower($tagetFileExt), $checkFileExt)) {
          return FALSE;
       }

       /* 기존 파일명과 중복 여부 확인  후 중복 파일이 있으면 파일명 변환*/
       if(file_exists(API_USER_IMG.'/'.$tagetFile)) 
         $tagetFile = pathinfo($tagetFile, PATHINFO_FILENAME).'_'.time().'.'.$tagetFileExt;

       if(!move_uploaded_file($_FILES['uploadFile']['tmp_name'], API_USER_IMG.'/'.$tagetFile)) {
           return FALSE;
       }

       return $tagetFile;
    }
    /** ==================================================================
     * License에 따른 최대 사용자 확인
     * TRUE : 최대 사용자 초과
     * FALSE : 최대 사용자 이하
     */
    private function checkMaxLimitedUser() {
       $maxUser = EnumTable::$attrModelSpec[$this->neptune_model][$this->neptune_type][2];
       if($this->util->GetRecordCount('User') >= $maxUser) return TRUE;
       else FALSE;   
    }
}
?>